"""
Concrete stress-strain relation functions sss

    .. uml::

        class stress_strain  <<(M,#FF7700)>> {
        .. functions ..
        + n(fck)
        + εc1(fck)
        + εc2(fck)
        + εc3(fck)
        + εcu1(fck)
        + εcu2(fck)
        + εcu3(fck)
        + stress_strain.σc_bilin(fck, αcc, γc, εc)
        + stress_strain.σc_design(fck, αcc, γc, εc)
        + stress_strain.σc_nl(fck, εc)
        }

"""

from . import strength
from . import elastic_deformation


def εc1(fck):
    """
    Compressive strain in the concrete at the peak stress (‰)

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expression:

        .. math::
            ε_{c1} = min(0.7\cdot f_{cm}^{0.31}, 2.8)

    """
    fcm = strength.fcm(fck)
    return min(2.8, 0.7*fcm**0.31)


def εc2(fck):
    """
    The strain at reaching the maximum strength (‰)

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expressions:

        .. math::
            \\begin{eqnarray}
                ε_{c2} & = & 2.0 & for & f_{ck} \le 50MPa \\\\
                ε_{c2} & = & 2.0 + 0.085(f_{ck}-50)^{0.53} & for & f_{ck} \ge 50MPa
            \\end{eqnarray}

    """
    if fck<50:
        _εc2 = 2.0
    else:
        _εc2 = 2.0+0.085*(fck - 50)**0.53

    return _εc2


def εc3(fck):
    """ (‰)

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expressions:

        .. math::
            \\begin{eqnarray}
                ε_{c3} & = & 1.75 & for & f_{ck} \le 50MPa \\\\
                ε_{c3} & = & 1.75 + 0.55(f_{ck}-50)/40 & for & f_{ck} \ge 50MPa
            \\end{eqnarray}

    """
    if fck<50:
        _εc3 = 1.75
    else:
        _εc3 = 1.75+0.55*(fck - 50)/40

    return _εc3


def εcu1(fck):
    """

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expressions:

        .. math::
            \\begin{eqnarray}
                ε_{cu1} & = & 3.5 & for & f_{ck} \le 50MPa \\\\
                ε_{cu1} & = & 2.8 + 27 \cdot ((98 - f_{cm})/100)^4 & for & f_{ck} \ge 50MPa
            \\end{eqnarray}

    """
    fcm = strength.fcm(fck)
    if fck<50:
        _εcu1 = 3.5
    else:
        _εcu1 = 2.8 + 27*((98-fcm)/100)**4

    return _εcu1


def εcu2(fck):
    """
    The ultimate strain

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expressions:

        .. math::
            \\begin{eqnarray}
                ε_{cu2} & = & 3.5 & for & f_{ck} \le 50MPa \\\\
                ε_{cu2} & = & 2.6 + 35 \cdot ((90 - f_{ck})/100)^4 & for & f_{ck} \ge 50MPa
            \\end{eqnarray}

    """
    if fck<50:
        _εcu2 = 3.5
    else:
        _εcu2 = 2.6 + 35*((90-fck)/100)**4

    return _εcu2


def εcu3(fck):
    """

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expressions:

        .. math::
            \\begin{eqnarray}
                ε_{cu2} & = & 3.5 & for & f_{ck} \le 50MPa \\\\
                ε_{cu2} & = & 2.6 + 35 \cdot ((90 - f_{ck})/100)^4 & for & f_{ck} \ge 50MPa
            \\end{eqnarray}

    """
    if fck<50:
        _εcu3 = 3.5
    else:
        _εcu3 = 2.6 + 35*((90-fck)/100)**4

    return _εcu3


def n(fck):
    """

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days

    Returns:
        float: Given using the expressions:

        .. math::
            \\begin{eqnarray}
                n & = & 2.0 & for & f_{ck} \le 50MPa \\\\
                n & = & 1.4 + 23.4 \cdot ((90 - f_{ck})/100)^4 & for & f_{ck} \ge 50MPa
            \\end{eqnarray}

    """
    if fck<50:
        _n = 2.0
    else:
        _n = 1.4 + 23.4*((90-fck)/100)**4

    return _n


def σc_nl(fck, εc):
    """
    Stress-strain relation for non-linear structural analyses

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days
        εc (float): concrete strain (‰)

    Returns:
        float: Given using the expression:

        .. math::
            \\begin{eqnarray}
                σ_{c} & = & f_{ctm}\dfrac{kη-η^2}{1+(k-2)η} \\\\
                where: & \\\\
                η & = & \dfrac{ε_c}{ε_{c1}} \\\\
                k & = & 1.05\cdot E_{cm} \cdot ε_{c1} / f_{cm}
            \\end{eqnarray}

    """
    _Ecm = elastic_deformation.Ecm(fck)
    _εc1 = εc1(fck)
    _fcm = strength.fcm(fck)

    η = εc / _εc1
    k = 1.05 * _Ecm * _εc1 / _fcm
    _σc = _fcm * (k * η - η * η)/ (1 + η * (k - 2))

    return _σc


def σc_design(fck, αcc, γc, εc):
    """
    Stress-strain relations for the design of cross-sections

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days
        αcc (float): Coefficient taking account of long term effects on the compressive strength
                     and of unfavourable effects resulting from the way the load is applied
        γc (float): Safety factor
        εc (float): concrete strain (‰)

    Returns:
        float: Given using the expression:

        .. math::
            \\begin{eqnarray}
                σ_{c} & = & f_{cd}\cdot(1-(1-\dfrac{ε_c}{ε_{c2}})^n) & for & 0\le ε_c \le ε_{c2} \\\\
                σ_{c} & = & f_{cd} & for & ε_{c2}\le ε_c \le ε_{cu2}
            \\end{eqnarray}

    """
    _fcd = strength.fcd(αcc, fck, γc)
    _εc2 = εc2(fck)
    _εcu2 = εcu2(fck)
    _n = n(fck)

    if εc < _εc2:
        _σc = _fcd * ( 1 - (1 - εc / _εc2)**_n)
    else:
        _σc = _fcd

    return _σc


def σc_bilin(fck, αcc, γc, εc):
    """

    Args:
        fck (float): Characteristic compressive cylinder strength of concrete at 28 days
        αcc (float): Coefficient taking account of long term effects on the compressive strength
                     and of unfavourable effects resulting from the way the load is applied
        γc (float): Safety factor
        εc (float): concrete strain (‰)

    Returns:
        float:

    """
    _fcd = strength.fcd(αcc, fck, γc)
    _εc3 = εc3(fck)

    if εc < _εc3:
        _σc = _fcd * εc / _εc3
    else:
        _σc = _fcd

    return _σc





import pandas as pd
import string

alphabet = string.ascii_uppercase


# Κόμβοι
def sap2d_nodes(_df_nodes):
    _str = 'TABLE:  "JOINT COORDINATES"\n'
    for row in _df_nodes.itertuples():
        _str += f'Joint={row.Node}   CoordSys=GLOBAL   CoordType=Cartesian   ' \
                f'XorR={row.X:.3f}   Y=0   Z={row.Z:.3f}   SpecialJt=No\n'
    return _str + '\n'


# Gridlines
def sap2d_gridlines(_df_nodes):
    xs = _df_nodes.X.unique()
    zs = _df_nodes.Z.unique()

    _str = 'TABLE:  "GRID LINES"\n'

    i = 0
    for x in xs:
        _str += f'   CoordSys=GLOBAL   AxisDir=X   GridID={alphabet[i]}   XRYZCoord={x}   LineType=Primary   ' \
                f'LineColor=Gray8Dark   Visible=Yes\n'
        i += 1

    _str += '   CoordSys=GLOBAL   AxisDir=Y   GridID=1   XRYZCoord=0   LineType=Primary   LineColor=Gray8Dark   ' \
            'Visible=Yes\n'

    i = 0
    for z in zs:
        _str += f'   CoordSys=GLOBAL   AxisDir=Z   GridID=Z{i}   XRYZCoord={z}   LineType=Primary   ' \
                f'LineColor=Gray8Dark   Visible=Yes\n'
        i += 1

    return _str + '\n'


# Στηρίξεις
def sap2d_node_restraints(_df_node_restraints):
    _str = 'TABLE:  "JOINT RESTRAINT ASSIGNMENTS"\n'
    for row in _df_node_restraints.itertuples():
        _str += f'   Joint={row.Node}   ' + \
                f'U1={row.U1}   U2={row.U2}   U3={row.U3}   ' \
                f'R1={row.R1}   R2={row.R2}   R3={row.R3}\n'.replace('=1', '=Yes').replace('=0', '=No')
    return _str + '\n'


# Διαφράγματα
def sap2d_diaphragms(_df_node_constraints):
    _str = 'TABLE:  "CONSTRAINT DEFINITIONS - DIAPHRAGM"\n'
    for row in _df_node_constraints.itertuples():
        _str += f'   Name=DIAPH{row.ID}   CoordSys=GLOBAL   Axis=Z   MultiLevel=No\n'
    return _str + '\n'


def sap2d_diaphragms_assignments(_df_node_constraints_assignments):
    _str = 'TABLE:  "JOINT CONSTRAINT ASSIGNMENTS"\n'
    for row in _df_node_constraints_assignments.itertuples():
        nodes = row.NODES.split(',')
        for node in nodes:
            _str += f'   Joint={node}   Constraint=DIAPH{row.ID}\n'
    return _str + '\n'


# Connectivity - Frame
def sap2d_frame_connectivity(_df_frame_connectivity):
    _str = 'TABLE:  "CONNECTIVITY - FRAME"\n'
    for row in _df_frame_connectivity.itertuples():
        _str += f'   Frame={row.FRAME}   JointI={row.NODEI}   JointJ={row.NODEJ}   IsCurved=No\n'
    return _str + '\n'


# Materials - General
def sap2d_materials_general(_df_materials_general):
    _str = 'TABLE:  "MATERIAL PROPERTIES 01 - GENERAL"\n'
    for row in _df_materials_general.itertuples():
        _str += f'   Material={row.MATERIAL_ID}   Type={row.TYPE}   SymType=Isotropic   TempDepend=No\n'
    return _str + '\n'


# Materials - Basic Mechanical Properties
def sap2d_materials_basic_mechanical(_df_materials_general):
    _str = 'TABLE:  "MATERIAL PROPERTIES 02 - BASIC MECHANICAL PROPERTIES"\n'
    for row in _df_materials_general.itertuples():
        _str += f'   Material={row.MATERIAL_ID}   UnitWeight=0   UnitMass=0   E1={row.E}   U12={row.POISSON}   A1=0\n'
    return _str + '\n'


def get_section_name(sect_geom, sect_id, _df_section_rect_elastic):
    if sect_geom == 'RECT':
        df = _df_section_rect_elastic.set_index("RECT_SECTION_ID")
        rect_name = df.at[sect_id, 'SECTION_NAME']
        return rect_name


# FRAME SECTION ASSIGNMENTS
def sap2d_frame_section_assignments(_df_frame_connectivity, _df_section_rect_elastic):
    _str = 'TABLE:  "FRAME SECTION ASSIGNMENTS"\n'
    for row in _df_frame_connectivity.itertuples():
        sect_id = get_section_name(row.SECT_GEOM, row.SECTION_ID, _df_section_rect_elastic)
        _str += f'   Frame={row.FRAME}   AutoSelect=N.A.   AnalSect={sect_id}   MatProp=Default\n'
    return _str + '\n'


# "FRAME SECTION PROPERTIES 01 - GENERAL"
def sap2d_frame_section_properties_general(_df_section_rect_elastic):
    _str = 'TABLE:  "FRAME SECTION PROPERTIES 01 - GENERAL"\n'
    for row in _df_section_rect_elastic.itertuples():
        _str += f'   SectionName={row.SECTION_NAME}   Material={row.MATERIAL}   Shape=Rectangular   ' \
                f't3={row.h:.3f}   t2={row.b:.3f}   ' \
                f'AMod={row.modArea:.3f}   A2Mod={row.modShear:.3f}   A3Mod={row.modShear:.3f}   ' \
                f'JMod={row.modTorsional:.3f}   I2Mod={row.modMoment:.3f}   I3Mod={row.modMoment:.3f}   ' \
                f'MMod=0   WMod=0\n'
    return _str + '\n'


# Load Patterns
def sap2d_load_patterns(_df_load_patterns):
    _str = 'TABLE:  "LOAD PATTERN DEFINITIONS"\n'
    for row in _df_load_patterns.itertuples():
        _str += f'   LoadPat={row.NAME}   DesignType={row.DesignType}   SelfWtMult=0\n'
    return _str + '\n'


# Load Cases
def sap2d_load_cases(_df_load_patterns):
    _str = 'TABLE:  "LOAD CASE DEFINITIONS"\n'
    _str += '   Case=MODAL   Type=LinModal   InitialCond=Zero   DesTypeOpt="Prog Det"   DesignType=OTHER   ' \
            'AutoType=None   RunCase=Yes\n'
    for row in _df_load_patterns.itertuples():
        _str += f'   Case={row.NAME}   Type=LinStatic   InitialCond=Zero   DesTypeOpt="Prog Det"   ' \
                f'DesignType={row.DesignType}   AutoType=None   RunCase=Yes\n'
    return _str + '\n'


# Load Cases - Static - Assignments
def sap2d_load_cases_static_assignments(_df_load_patterns):
    _str = 'TABLE:  "CASE - STATIC 1 - LOAD ASSIGNMENTS"\n'
    for row in _df_load_patterns.itertuples():
        _str += f'   Case={row.NAME}   LoadType="Load pattern"   LoadName={row.NAME}   LoadSF=1\n'
    return _str + '\n'


# Masses
def sap2d_masses():
    _str = 'TABLE:  "MASSES 1 - MASS SOURCE"\n'
    _str += '   MassFrom=Elements\n'
    return _str + '\n'


def sap2d_masses_nodal(_df_masses_nodal):
    _str = 'TABLE:  "JOINT ADDED MASS ASSIGNMENTS"\n'
    for row in _df_masses_nodal.itertuples():
        _str += f'   Joint={row.NODE}   CoordSys=Global   ' \
                f'Mass1={row.UX:.3f}   Mass2={row.UY:.3f}   Mass3={row.UZ:.3f}   ' \
                f'MMI1={row.RX:.3f}   MMI2={row.RY:.3f}   MMI3={row.RZ:.3f}\n'
    return _str + '\n'


# Loads - Nodal
def sap2d_loads_nodal(_df_loads_nodal):
    _str = 'TABLE:  "JOINT LOADS - FORCE"\n'
    for row in _df_loads_nodal.itertuples():
        _str += f'   Joint={row.NODE}   LoadPat={row.LOAD_PATTERN}   CoordSys=Global   ' \
                f'F1={row.FX:.3f}   F2={row.FY:.3f}   F3={row.FZ:.3f}   ' \
                f'M1={row.MX:.3f}   M2={row.MY:.3f}   M3={row.MZ:.3f}\n'
    return _str + '\n'


# Loads - Distributed
def sap2d_loads_dist(_df_loads_dist_gravity):
    _str = 'TABLE:  "FRAME LOADS - DISTRIBUTED"\n'
    for row in _df_loads_dist_gravity.itertuples():
        _str += f'   Frame={row.FRAME}   LoadPat={row.LOAD_PATTERN}   CoordSys=GLOBAL   Type=Force   Dir=Gravity   ' \
                f'DistType=RelDist      RelDistA=0   RelDistB=1   ' \
                f'FOverLA={row.VALUE:.3f}   FOverLB={row.VALUE:.3f} \n'
    return _str + '\n'



from dataclasses import dataclass, field, fields
from .results_display_older import *
from typing import List

@dataclass
class NamedLogTable:
    name: str
    dict_list: List[dict] = field(default_factory=list)

    def to_markdown(self):
        return log_markdown_table(self.dict_list)

    def to_panda_dataframe(self):
        return log_dataframe(self.dict_list)


@dataclass
class NamedLogText:
    name: str
    text_list: List[str] = field(default_factory=list)


@dataclass
class Output:
    # log_text: List[str] = field(default_factory=list)
    log_text: NamedLogText = NamedLogText('main output text')
    log_table: NamedLogTable = NamedLogTable('main output table')

    tables: List[NamedLogTable] = field(default_factory=list)
    # Μπορώ να το κάνω να δίνει τα αποτελέσματα αρχικά λίστα από pandas και μετά σε αρχείο excel
    # με sheetname το όνομα κάθε NamedLogTable

    texts: List[NamedLogText] = field(default_factory=list)
    # Ομοίως σε json???

    # def __post_init__(self):


    @property
    def show_log_text(self):
        return '\n'.join(self.log_text)




from dataclasses import dataclass, field

@dataclass
class RCTypology:
    name: str = None
    code: str = field(init=False)
    storeys: int = field(init=False)
    height_level: str = field(init=False)
    structural_system: str = field(init=False)
    infill_pattern: str = field(init=False)
    code_level: str = field(init=False)

    @classmethod
    def from_old_analyses_name(cls, old_name):
        spl = old_name.split('_')

        cls.storeys = int(spl[0])

        if spl[1] == 'du':
            cls.structural_system = 'dual'
        elif spl[1] == 'fr':
            cls.structural_system = 'frame'
        else:
            cls.structural_system = 'oops!'

        if spl[2] == '59':
            cls.code = 'bd59'
        elif spl[2] == '84':
            cls.code = 'pa84'
        elif spl[2] == 'eak':
            cls.code = 'eak'
        else:
            cls.code = 'oops'

        if spl[3] == 'bare':
            cls.infill_pattern = 'bare'
        elif spl[3] == 'inf':
            cls.infill_pattern = 'infilled'
        elif spl[3] == 'pil':
            cls.infill_pattern = 'soft_storey'
        else:
            cls.infill_pattern = 'oops'

        return cls

    @classmethod
    def from_riskue_name(cls, riskue_name):
        strsys_inf = riskue_name[2:len(riskue_name) - 2]
        if strsys_inf == '1':
            cls.structural_system = 'frame'
            cls.infill_pattern = 'bare'
        elif strsys_inf == '3.1':
            cls.structural_system = 'frame'
            cls.infill_pattern = 'infilled'
        elif strsys_inf == '3.2':
            cls.structural_system = 'frame'
            cls.infill_pattern = 'soft_storey'
        if strsys_inf == '4.1':
            cls.structural_system = 'dual'
            cls.infill_pattern = 'bare'
        elif strsys_inf == '4.2':
            cls.structural_system = 'dual'
            cls.infill_pattern = 'infilled'
        elif strsys_inf == '4.3':
            cls.structural_system = 'dual'
            cls.infill_pattern = 'soft_storey'
        else:
            cls.structural_system = 'oops'
            cls.infill_pattern = 'oops'

        cls.height_level = riskue_name[-2]
        cls.code_level = riskue_name[-1]

        return cls

    @staticmethod
    def get_riskue_height_level_from_storeys(storeys):
        _hl = ''
        if storeys <=3:
            _hl = 'L'
        elif storeys >=8:
            _hl = 'H'
        else:
            _hl = 'M'
        return _hl

    @staticmethod
    def get_riskue_design_level(code):
        _dl = ''
        if code == 'bd59':
            _dl = 'L'
        elif code == 'pa84':
            _dl = 'M'
        elif code == 'eak':
            _dl = 'H'
        else:
            _dl = 'N'
        return _dl


    @property
    def riskue_name(self) -> str:
        print('working!!!')
        tmp_code = 'RC'

        if self.structural_system == 'frame':
            if self.infill_pattern == 'infilled':
                tmp_code += '3.1'
            elif self.infill_pattern == 'soft_storey':
                tmp_code += '3.2'
            elif self.infill_pattern == 'bare':
                tmp_code += '1'

        elif self.structural_system == 'dual':
            if self.infill_pattern == 'infilled':
                tmp_code += '4.2'
            elif self.infill_pattern == 'soft_storey':
                tmp_code += '4.3'
            elif self.infill_pattern == 'bare':
                tmp_code += '4.1'

        tmp_code += self.get_riskue_height_level_from_storeys(self.storeys)
        tmp_code += self.get_riskue_design_level(self.code)

        return tmp_code




import numpy as np
import math
import pandas as pd
from dataclasses import dataclass, astuple, field
from typing import List
from goto import with_goto
from . import rcinput
from .concrete_models import Kappos91, KentPark71
from .rcsubs import BUCKL, PLCENT, STRESS, STRHRD, CFRCR, CFRCS, YLDDUC, DUCTLE, SHEAR
from .rcsubs import init_arrays_no_length


@dataclass
class Rccola:
    outtxt: List[str] = field(default_factory=list)
    outfilename: str = 'pyrcout'
    input: rcinput.RccolaInput = None
    uniaxial_bending_ini_dfs: List[pd.DataFrame] = field(default_factory=list)
    uniaxial_bending_conf_dfs: List[pd.DataFrame] = field(default_factory=list)
    uniaxial_bending_dfs: List[pd.DataFrame] = field(default_factory=list)

    def write_out(self):
        file = open(self.outfilename, 'w')
        file.write('\n'.join(self.outtxt))
        file.close()

    def calc(self):
        self.main()

        NLOAD = self.input.second_row.NLOAD
        if NLOAD == 0:
            NLOAD = 15

        for i, (df_ini, df_conf) in enumerate(zip(self.uniaxial_bending_ini_dfs, self.uniaxial_bending_conf_dfs)):
            df_ini_sorted = df_ini.sort_values(by=['εc'])
            df_conf_sorted = df_conf.sort_values(by=['εc'])

            df_join1 = df_ini_sorted.where(df_ini_sorted['εc']<=0.0035).dropna()
            df_join2 = df_conf_sorted.where(df_conf_sorted['εc']>0.0035).dropna()

            self.uniaxial_bending_dfs.append(df_join1.append(df_join2))
            


    @with_goto
    def main(self):
        # Read input data
        TYPEC, TYPES, ISPALL, ICSTR, ICSTRS, NPTS, NPTSS, ITCONF, ISSTL, IECU, ISSTR, IGRAPH = astuple(
            self.input.first_row)
        NLOAD, NLAYER, NDATA1, NDATA2, NSTRNI, NSTRNC, NITT = astuple(
            self.input.second_row)
        T, B, TC, BC = astuple(self.input.conc_sect_info)
        if TYPES != 3:
            GT, NB1, AS1, NB2, AS2, NBAR, AF = astuple(
                self.input.reinf_place_info_TYPES12)
        else:
            NL, NBAR, YS, AS, NBS, IBUC, YSS, ASS, NBSS = astuple(
                self.input.reinf_place_info_TYPES3)
        FYS, YMS, ESH, YSH, ESU, FSU, ESDYN = astuple(
            self.input.long_reinf_props)
        FYSTRP, ASSTRP, S, XL, XSTRP, ESUSTRP = astuple(
            self.input.trans_reinf_info)
        # Στα παρακάτω ορίζεται 2η φορά το S...δεν πειράζε...αλλά δεν υπάρχει και λόγος
        if ICSTR in [2, 3, 4, 8]:
            FCJ, S, PPP = astuple(self.input.conc_props_ICSTR2348)
        if ICSTRS == 2:
            FCJS, SS, PPX, NSP = astuple(
                self.input.conc_unconfined_props_ICSTRS2)
        STRN = self.input.strains
        if TYPEC == 3:
            AA, N1, A1, AAc, N1c = astuple(
                self.input.geometry_conc_crossec_info)

        abc = ['', 'A', 'B', 'C']
        default_norm_axials = np.array(
            [1., 0.8, 0.6, 0.5, 0.4, 0.35, 0.3, 0.25, 0.2, 0.15, 0.1, 0.])

        axial_loads = self.input.axials

        if axial_loads is None or len(axial_loads) == 0:
            axial_loads = np.array([])

        if NLOAD == 0:
            NLOAD = 15

        if NLAYER == 0:
            NLAYER = 150

        if NITT == 0:
            NITT = 15

        if NSTRNC < NSTRNI:
            NSTRNC = NSTRNI

        if NSTRNC + NSTRNI == 0:
            STRN = [0, 0.0005]
            STRN.extend([0.0015 + 0.0005 * i for i in range(0, 8)])
            STRN.extend([0.006 + 0.001 * i for i in range(0, 5)])
            STRN.extend([0.015 + 0.005 * i for i in range(0, 8)])
            NSTRNI = 7
            NSTRNC = 22

        JJ = NSTRNI + 3
        JJJ = NSTRNC + 3

        # Initialize some variables ....μάλλον δεν χρειάζεται
        AAC = 0.
        DUCMXI = 0.0
        DUCMXC = 0.0

        FCJ1 = 0.0
        ECULT = 0.0

        A4I = 0.0
        A4C = 0.0
        A4S = 0.0
        A4SS = 0.0

        # Initialize arrays
        EC = []
        FC = []
        ECS = []
        FCS = []

        VI = np.zeros(len(STRN) + 4)

        XPCALC = np.zeros(NLOAD + 1)

        VMI = np.zeros((NLOAD + 1, len(STRN) + 4))
        XXMI = np.zeros((NLOAD + 1, len(STRN) + 4))
        PHII = np.zeros((NLOAD + 1, len(STRN) + 4))
        DUCTI = np.zeros((NLOAD + 1, len(STRN) + 4))

        VC = np.zeros(len(STRN) + 4)

        VMC = np.zeros((NLOAD + 1, len(STRN) + 4))
        XXMC = np.zeros((NLOAD + 1, len(STRN) + 4))
        PHIC = np.zeros((NLOAD + 1, len(STRN) + 4))
        DUCTC = np.zeros((NLOAD + 1, len(STRN) + 4))

        A = np.zeros(NLAYER + 1)
        Y = np.zeros(NLAYER + 1)
        AC = np.zeros(NLAYER + 1)
        YC = np.zeros(NLAYER + 1)

        # YS = np.zeros(NL)
        # AS = np.zeros(NL)
        # NBS = np.zeros(NL)
        # IBUC = np.zeros(NL)
        # if TYPES == 3:
        #     AA = np.zeros(NL)

        ASS = np.zeros(ISSTL + 1)
        YSS = np.zeros(ISSTL + 1)

        ES = np.zeros(32)
        AREA = np.zeros(32)
        CCC = np.zeros(32)
        EES = np.zeros(32)
        ECC = np.zeros(32)
        DUCT1 = np.zeros(32)
        O = np.zeros(32)

        if ISPALL == 1:
            ITCONF = 0

        self.outtxt.append(
            " R E I N F O R C E D   C O N C R E T E   C O L U M N   A N A L Y S I S  ")
        self.outtxt.append(
            " PROGRAM RCCOLA-90    PROPERTIES OF REINFORCED CONCRETE CROSS-SECTIONS")
        self.outtxt.append("")
        self.outtxt.append("")
        self.outtxt.append(
            " ***********************************************************************")
        self.outtxt.append(" Title ")
        self.outtxt.append(
            " ***********************************************************************")
        self.outtxt.append("")
        self.outtxt.append("")

        if TYPEC == 1:
            self.outtxt.append(" RECTANGULAR CROSS-SECTION")
            self.outtxt.append(f"   DEPTH = {T:10.4f}  m")
            self.outtxt.append(f"   WIDTH = {B:10.4f}  m")
            self.outtxt.append("")
            self.outtxt.append("")

        if TYPES == 1:
            self.outtxt.append(" RECTANGULAR STEEL PLACEMENT")
            self.outtxt.append(f"   GT               ={GT:10.4f}  m")
            self.outtxt.append(f"   TC               ={TC:10.4f}  m")
            self.outtxt.append(f"   BC               ={BC:10.4f}  m")
            self.outtxt.append("   MAIN REINFORCING")
            self.outtxt.append(f"     NUMBER OF BARS ={NB1:10.4f}")
            self.outtxt.append(f"     BAR AREA       ={AS1:10.4f} cm**2")
            self.outtxt.append("   SIDE REINFORCING")
            self.outtxt.append(f"     NUMBER OF BARS ={NB2:10.4f}")
            self.outtxt.append(f"     BAR AREA       ={AS2:10.4f} cm**2")
            self.outtxt.append("")
            self.outtxt.append("")

            AS1 = 0.0001 * AS1
            AS2 = 0.0001 * AS2
            ASTOT = NB1 * AS1 + NB2 * AS2

        if TYPES == 3:
            self.outtxt.append(" GENERAL STEEL PLACEMENT")
            self.outtxt.append("")
            self.outtxt.append("   DEPTH  NO OF BARS  ' BAR AREA     IBUC")
            self.outtxt.append("    (m)                 (cm**2)")
            for i in range(0, NL):
                self.outtxt.append(
                    f"{YS[i]:8.3f}{NBS[i]:8}{AS[i]:14.4f}{IBUC[i]}")

            if BC <= 0:
                BC = B

            if NL != 0:
                ASTOT = 0.0
                for i in range(0, NL):
                    if NBS[i] == 0:
                        NBS[i] = 1
                    AS[i] = 0.0001 * AS[i] * NBS[i]
                    ASTOT += AS[i]

            if ISSTL != 0:
                self.outtxt.append(" GENERAL STEEL PLACEMENT")
                self.outtxt.append("   DEPTH      AREA")
                self.outtxt.append("    (m)     (cm**2)")
                for i in range(0, ISSTL):
                    self.outtxt.append(f"{YSS[i]:8.3f}{ASS[i]:10.4f}")
                    if NBSS[i] == 0:
                        NBSS[i] = 1
                    ASS[i] = 0.0001 * ASS[i] * NBSS[i]

        FYS1 = FYS
        ESY = FYS / YMS
        if ESH < ESY:
            ESH = ESY
        R = ESU - ESH
        YMAX = FSU - FYS
        if YSH == 0.:
            YSH = 3. * YMAX / R

        # IF (ISSTR.GT.0.) CALL STRST(FYS,YMS,YSH,ESU,ESDYN,ESY,FSU)

        C1 = (YSH - 2. * YMAX / R) / (R * R)
        C2 = 3.0 * (YMAX - 0.6667 * YSH * R) / (R * R)

        self.outtxt.append(" MATERIAL PROPERTIES -- STEEL")
        self.outtxt.append(f"     YIELD STRESS        ={FYS:10.1f} MPa")
        self.outtxt.append(f"     YOUNGS MODULUS      ={YMS:10.1f} MPa")
        self.outtxt.append("     STRAIN AT ONSET OF")
        self.outtxt.append(f"        STRAIN HARDENING ={ESH:10.6f} m/m")
        self.outtxt.append("     STRAIN HARDENING")
        self.outtxt.append(f"        MODULUS          ={YSH:10.2f} MPa")
        self.outtxt.append(f"     MAXIMUM STRAIN      ={ESU:10.6f} m/m")
        self.outtxt.append(f"     MAXIMUM STRESS      ={FSU:10.1f} MPa")
        self.outtxt.append("")
        self.outtxt.append("")

        if FYSTRP == 0:
            FYSTRP = FYS
        if XL == 0.:
            XL = 3.
        if XSTRP == 0.:
            XSTRP = 2.
        if ESUSTRP == 0.:
            ESUSTRP = ESU

        self.outtxt.append(" TRANSVERSE REINFORCEMENT")
        self.outtxt.append(f"     YIELD STRESS   ={FYSTRP:10.2f} MPa")
        self.outtxt.append(f"     AREA/SPACING   ={ASSTRP:10.4f} cm**2")
        self.outtxt.append(f"     NO OF LEGS     ={XSTRP:10.2f}")
        self.outtxt.append(f"     SPACING        ={S:10.4f} m")
        self.outtxt.append(f"     COLUMN LENGTH  ={XL:10.4f} m")
        self.outtxt.append("")
        self.outtxt.append("")

        ASSTRP = ASSTRP * 0.0001
        F = FYSTRP * ASSTRP * 1000. / S
        A1STRP = ASSTRP / XSTRP

        self.outtxt.append(" MATERIAL PROPERTIES -- CONCRETE")
        self.outtxt.append("")

        if ICSTR == 8:
            if NBAR == 0:
                α_kap91 = 0.55
                β_kap91 = 0.75
            elif NBAR == 1.0:
                α_kap91 = 1.0
                β_kap91 = 1.0
            elif NBAR >= 2.0:
                α_kap91 = 1.25
                β_kap91 = 1.0
            else:
                α_kap91 = 1.0
                β_kap91 = 1.0
                print('Check NBAR')

            kap91 = Kappos91(fc=FCJ,
                             fy=FYS,
                             bc=BC,
                             s=S,
                             ρw=PPP,
                             α=α_kap91,
                             β=β_kap91,
                             εc0=0.002)

            FCJ1 = kap91.fcc
            ECULT = kap91.εcult

            self.outtxt.append(f"    Kappos (1990) model")
            self.outtxt.append(f"")
            self.outtxt.append(f"     Cylinder strength = {FCJ:10.4f} MPa")
            self.outtxt.append(f"     Tie spacing       = {S:10.4f}   m ")
            self.outtxt.append(f"     Tie/core ratio    = {PPP:10.4f} vol/vol")
            self.outtxt.append(f"     No of interm. bars= {NBAR:10.4f}")
            self.outtxt.append(f"     Stength factor    = {kap91.k:10.4f}")
            self.outtxt.append(f"     Slope Z           = {kap91.u:10.4f}")
            self.outtxt.append(f"")
            self.outtxt.append(f"     Ultimate strain   = {ECULT:10.5f}")
            self.outtxt.append(f"")
            self.outtxt.append(f"")

            EC = np.linspace(0., kap91.εcc0, 11)
            EC = np.append(EC, [kap91.ε25c, kap91.ε25c + 0.05])
            FC = np.array([kap91.σc(_ε) for _ε in EC])

        self.outtxt.append(f"         STRAIN    STRESS")
        self.outtxt.append(f"        ( m/m )     (MPa)")
        for i, ε in enumerate(EC):
            self.outtxt.append(f"{ε:15.5f}{FC[i]:10.3f}")

        if FCJ1 == 0.0:
            FCJ1 = FCJ

        #   *************************** Unconfined concrete model **********************************
        # 220
        if ISPALL != 0:
            if ICSTRS == 1:  # User defined
                pass
            elif ICSTRS == 2:  # Kent/Park (1971)
                kentpark71 = KentPark71(fc=FCJS,
                                        bc=BC,
                                        s=SS,
                                        ρw=PPX,
                                        nsp=NSP,
                                        εc0=0.002)

                ECS = np.linspace(0., kentpark71.εc0, 11)
                if NSP == 0:
                    ECS = np.append(ECS, [kentpark71.ε20c, 0.1])
                else:
                    ECS = np.append(
                        ECS, [kentpark71.ε50u, kentpark71.ε50u + 0.00001, 0.05])
                FCS = np.array([kentpark71.σc(_ε) for _ε in ECS])

                if NSP == 1:
                    FCS[len(FCS) - 3] = 0.5 * kentpark71.fc

                self.outtxt.append(f"")
                self.outtxt.append(f"")
                self.outtxt.append(f"    Park/Kent (1971) model ")
                self.outtxt.append(f"     MAXIMUM STRESS = {FCJS:10.4f} MPa")
                self.outtxt.append(f"     TIE SPACING    = {S:10.4f} m")
                self.outtxt.append(
                    f"     TIE/CORE RATIO = {PPX:10.6f} vol/vol")
                self.outtxt.append(f"     SPALLING CODE  = {NSP}")
                self.outtxt.append(f"     Z = {kentpark71.z:10.4f}")

                self.outtxt.append(f"")
                self.outtxt.append(f"         STRAIN    STRESS")
                self.outtxt.append(f"        ( m/m )     (MPa)")
                for i, ε in enumerate(ECS):
                    self.outtxt.append(f"{ε:15.6f}{FCS[i]:10.3f}")

            elif ICSTRS == 3:  # Carreira/Chu (1985)
                pass
            else:
                print('Wrong ICSTRS value for unconfined concrete')

        #   ******************** Calculate ultimate strain and Young's modulus for concrete ****************************
        # 260
        if ECULT == 0.0:
            PPW = PPP * FYSTRP / FCJ
            if AF == 0.:
                if NBAR == 0.0:
                    AF = 0.6667
                else:
                    AF = 1.3333

            ECULT = 0.0035 + 0.025 * (1 + 0.65 * NBAR ** 0.6667) * PPW * AF

        ECMAND = 0.004 + 1.4 * PPP * FYSTRP * ESUSTRP / FCJ1

        ECBUC, bucltxt = BUCKL(AS1=AS1 if TYPES != 3 else None,
                               AS2=AS2 if TYPES != 3 else None,
                               AS=[] if TYPES != 3 else AS,
                               YS=[] if TYPES != 3 else YS,
                               NBS=[] if TYPES != 3 else NBS,
                               IBUC=[] if TYPES != 3 else IBUC,
                               A1STRP=A1STRP,
                               YSH=YSH,
                               BC=BC,
                               TC=TC,
                               S=S,
                               FSU=FSU,
                               FYS=FYS,
                               FYSTRP=FYSTRP,
                               ESH=ESH)

        # if TYPES != 3:
        #     ECBUC, bucltxt = BUCKL(AS1=AS1,
        #                            AS2=AS2,
        #                            AS=[],
        #                            YS=[],
        #                            NBS=[],
        #                            IBUC=[],
        #                            A1STRP=A1STRP,
        #                            YSH=YSH,
        #                            BC=BC,
        #                            TC=TC,
        #                            S=S,
        #                            FSU=FSU,
        #                            FYS=FYS,
        #                            FYSTRP=FYSTRP,
        #                            ESH=ESH)
        # else:
        #     ECBUC, bucltxt = BUCKL(AS1=None,
        #                            AS2=None,
        #                            AS=AS,
        #                            YS=YS,
        #                            NBS=NBS,
        #                            IBUC=IBUC,
        #                            A1STRP=A1STRP,
        #                            YSH=YSH,
        #                            BC=BC,
        #                            TC=TC,
        #                            S=S,
        #                            FSU=FSU,
        #                            FYS=FYS,
        #                            FYSTRP=FYSTRP,
        #                            ESH=ESH)

        if ECBUC < 0.004:
            ECBUC = 0.004

        self.outtxt.append(f"")
        self.outtxt.extend(bucltxt)

        ECFAIL = min(ECULT, ECMAND, ECBUC)
        if IECU == 0:
            ECULT = ECFAIL
            # ECULTconsidered = ECFAIL
        else:
            ECULT = max(ECULT, ECMAND, ECBUC)
            # ECULTconsidered = max(ECULT, ECMAND, ECBUC)

        self.outtxt.append(f"")
        self.outtxt.append(f"  No of intermediate bars per side :{NBAR}")
        self.outtxt.append(
            f"  Strain at first hoop fracture    :{ECMAND:12.6f}")
        self.outtxt.append(
            f"  Strain at buckling of long. bars :{ECBUC:12.6f}")
        self.outtxt.append(f"")
        self.outtxt.append(
            f"  Ultimate conc. strain considered :{ECULT:12.6f}")
        # self.outtxt.append(f"  Ultimate conc. strain considered :{ECULTconsidered:12.6f}")

        if ICSTR == 1:
            FCJ = max(FC)

        EEC = (FC[1] - FC[0]) / (EC[1] - EC[0])
        if EEC < 25000:
            EEC = 21500 * (FCJ / 10) ** 0.3333

        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f" YOUNGS MODULUS FOR CONCRETE={EEC:10.1f} MPa")

        RECS = YMS / EEC - 1.0
        G = 5 * math.sqrt(FCJ * 1000)

        self.outtxt.append(f" EQUILIBRIUM CALCULATED FOR THE FOLLOWING")
        self.outtxt.append(f" CONCRETE COMPRESSIVE STRAINS")
        self.outtxt.append(f"")
        self.outtxt.append(f"         M         STRAIN")
        self.outtxt.append(f"")

        # for idx, val in enumerate(STRN):
        #     self.outtxt.append(f"{idx}{val:15.6f}")

        for i in range(0, NSTRNI + 1):
            self.outtxt.append(f"{i}{STRN[i]:15.6f}")

        for i in range(NSTRNI + 1, NSTRNC + 1):
            self.outtxt.append(f"{i}{STRN[i]:15.6f}  CONFINED SECTION ONLY")

        DLAYER = 1.0 / NLAYER
        DD = 0.5 * (1.0 + DLAYER)

        if TYPEC == 1:  # Rectangular
            AG = T * B
            AII = B * T ** 3 / 12.

            AGC = TC * BC
            AIIC = BC * TC ** 3. / 12.

        elif TYPEC == 2:  # Circular
            AII = 0.0
            RR = T * T / 4.
            AG = math.pi * RR
            for i in range(0, NLAYER):
                Y[i] = T * (DD - DLAYER * i)
                A[i] = 2.0 * DLAYER * T * math.sqrt(RR - Y[i] * Y[i])
                AII = AII + A[i] * Y[i] * Y[i]

            RRC = TC * TC / 4.0
            AGC = math.pi * RRC
            AIIC = 0.0
            for i in range(0, NLAYER):
                YC[i] = TC * (DD - DLAYER * i)
                AC[i] = 2.0 * DLAYER * TC * math.sqrt(RRC - YC[i] * YC[i])
                AIIC = AIIC + AC[i] * YC[i] * YC[i]

        else:  # Arbitrary
            AG = 0.0
            AGC = 0.0
            A4 = 0.0
            A4C = 0.0
            A4S = 0.0
            A4SS = 0.0
            AST = 0.0
            AAS = 0.0
            AII = 0.0
            AIIC = 0.0
            AIIS = 0.0
            AIISS = 0.0
            ASTLC = 0.0

            DLAYER = T / float(NLAYER)
            DD = 0.5 * (1.0 + DLAYER / T)
            MM = 0

            if ISPALL != 1:
                A1 = list(np.zeros(NDATA1))

            for i in range(0, NDATA1):
                AA[i] = AA[i] * DLAYER
                A1[i] = A1[i] * DLAYER
                aa = AA[i]
                a1 = A1[i]

                for j in range(0, N1[i]):
                    MM = MM + 1
                    A[MM] = aa
                    AC[MM] = a1
                    AG = AG + aa + a1
                    AGC = AGC + aa
                    Y[MM] = T * DD - DLAYER * float(MM)
                    YC[MM] = Y[MM]
                    BB = (aa + a1) * Y[MM]
                    BB1 = aa * Y[MM]
                    AII = AII + BB * Y[MM]
                    AIIC = AIIC + BB1 * YC[MM]
                    A4C = A4C + BB1
                    A4 = A4 + BB
            A4I = A4

            for i in range(0, NL):
                if YS[i] <= 0.:
                    ASTLC = ASTLC + AS[i]
                    AAC = AAC + AS[i] * YS[i]
                AST = AST + AS[i]
                aa = AS[i] * YS[i] * RECS
                AIIS = AIIS + aa * YS[i]
                A4S = A4S + aa
            AAA = AG + AST * RECS
            ASTLI = ASTLC
            AAI = AAC
            if ISPALL != 0:
                AAAC = AGC + AST * RECS
                YBAR = (A4C + A4S) / AAAC
                AIIC = AIIC + AIIS
                AIIC = AIIC + AAAC * YBAR * YBAR
            if ISSTL != 0:
                for i in range(0, ISSTL):
                    if YSS[i] < 0:
                        ASTLI = ASTLI + ASS[i]
                        AAI = AAI + ASS[i] * YSS[i]
                    AAS = AAS + ASS[i]
                    aa = ASS[i] * YSS[i] * RECS
                    AIISS = AIISS + aa * YSS[i]
                    A4SS = A4SS + aa
                AAA = AAA + AAS * RECS
            A4 = A4 + A4S + A4SS
            YBAR = A4 / AAA
            AII = AII + AIIS + AIISS
            AII = AII + AAA * YBAR * YBAR

            if ITCONF != 0:
                AGC = 0.0
                A4 = 0.0
                AIIC = 0.0

                DLAYER = TC / float(NLAYER)
                DD = 0.5 * (1.0 + DLAYER / TC)
                MM = 0

                for i in range(0, NDATA2):
                    AAc[i] = AAc[i] * DLAYER
                    aac = AAc[i]

                    for j in range(0, N1c[i]):
                        MM += 1
                        AC[MM] = aac
                        AGC = AGC + aac
                        YC[MM] = TC * DD - DLAYER * float(MM)
                        BB = aac * YC[MM]
                        AIIC = AIIC + BB * YC[MM]
                        A4 = A4 + BB
                A4C = A4
                AAAC = AGC + AST * RECS
                AIIC = AIIC + AIIS
                A4 = A4 + A4S
                YBAR = A4 / AAAC
                AIIC = AIIC + AAAC * YBAR * YBAR

            self.outtxt.append(f"")
            self.outtxt.append(f"")
            self.outtxt.append(f" GENERAL CONCRETE SECTION GEOMETRY")
            self.outtxt.append(f"")

            if ITCONF != 1:
                if ISPALL != 1:
                    self.outtxt.append(f" LAYER          DEPTH           AREA")
                    self.outtxt.append(f"")
                    for i in range(1, NLAYER + 1):
                        self.outtxt.append(f"{i:6}{Y[i]:15.5f}{A[i]:15.5f}")
                else:
                    self.outtxt.append(
                        f"      --------CONFINED CORE--------- -------UNCONFINED COVER-------")
                    self.outtxt.append(
                        f" LAYER          DEPTH           AREA          DEPTH           AREA")
                    self.outtxt.append(
                        f"                 (m)           (m**2)          (m)           (m**2)")
                    self.outtxt.append(f"")
                    for i in range(1, NLAYER + 1):
                        self.outtxt.append(
                            f"{i:6}{Y[i]:15.5f}{A[i]:15.5f}{YC[i]:15.5f}{AC[i]:15.5f}")
            else:
                self.outtxt.append(
                    f"      -------INITIAL SECTION-------- -------CONFINED SECTION-------")
                self.outtxt.append(
                    f" LAYER          DEPTH           AREA          DEPTH           AREA")
                self.outtxt.append(f"")
                for i in range(1, NLAYER + 1):
                    self.outtxt.append(
                        f"{i:6}{Y[i]:15.5f}{A[i]:15.5f}{YC[i]:15.5f}{AC[i]:15.5f}")

        # C
        # C     SECTION PROPERTIES - STEEL
        # C
        if TYPES == 1:  # Rectangular
            NL = round(NB2 / 2 + 2)
            YS = np.zeros(NL + 1)
            AS = np.zeros(NL + 1)

            YS[0] = GT / 2
            AS[0] = NB1 * AS1 / 2

            YS[NL - 1] = -YS[0]
            AS[NL - 1] = AS[0]

            if NL != 2:
                NNL = NL - 1
                DY = float(GT / NNL)

                for i in range(1, NNL):
                    YS[i] = YS[i - 1] - DY
                    AS[i] = 2. * AS2

            AII1 = AII
            for i in range(0, NL):
                AII = AII + AS[i] * RECS * YS[i] * YS[i]

        elif TYPES == 2:  # Circular
            THETA = 2. * math.pi / NB1
            AII1 = AII
            YS = np.zeros(NB1)
            AS = np.zeros(NB1)
            NL = NB1
            for i in range(0, NL):
                YS[i] = GT * math.cos(THETA * (float(i) - 1.)) / 2.0
                AS[i] = AS1
                AII = AII + AS1 * YS[i] * YS[i] * RECS

        if TYPES == 1 or TYPES == 2:
            DAII = AII - AII1
            AIIC = AIIC + DAII
            AAS = 0.0

            if ISSTL != 0:
                for i in range(0, ISSTL):
                    AAS = AAS + ASS[i]
                    AII = AII + ASS[i] * YSS[i] * YSS[i] * RECS

            AST = NB1 * AS1 + NB2 * AS2
            AAA = AG + (AST + AAS) * RECS
            AAAC = AGC + AST * RECS

            if S > 0.:
                ASTLI = (AST + AAS) / 2.0
                ASTLC = AST / 2.0

                for i in range(0, NL):
                    if YS[i] < 0:
                        AAC = AAC + YS[i] * AS[i]

                AAI = AAC

                if ISSTL != 0:
                    for i in range(0, ISSTL):
                        if YSS[i] <= 0.0:
                            AAI = AAI + YSS[i] * ASS[i]

        RG = math.sqrt(AII / AAA)

        if TYPES == 2:
            NBOTL = NL / 2. + 1
        else:
            NBOTL = NL

        AST1 = AST

        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f" GROSS SECTION")
        self.outtxt.append(f" TRANSFORMED AREA      ={AAA:15.6f}  m**2")
        self.outtxt.append(f" MOMENT OF INERTIA     ={AII:15.6f}  m**4")
        self.outtxt.append(f" RADIUS OF GYRATION    ={RG:15.4f}  m")

        if ITCONF != 0:
            RGC = math.sqrt(AIIC / AAAC)
            self.outtxt.append(f"")
            self.outtxt.append(f"")
            self.outtxt.append(f" CONFINED SECTION")
            self.outtxt.append(f" TRANSFORMED AREA      ={AAAC:15.6f}  m**2")
            self.outtxt.append(f" MOMENT OF INERTIA     ={AIIC:15.6f}  m**4")
            self.outtxt.append(f" RADIUS OF GYRATION    ={RGC:15.4f}  m")

        DI = -AAI / ASTLI + T / 2.0
        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f" PARAMETERS FOR SHEAR STRENGTH EVALUATION")
        self.outtxt.append(f"    BASED ON REINFORCEMENT ON TENSILE")
        self.outtxt.append(f"    SIDE OF SECTION MID-DEPTH")
        self.outtxt.append(f"")
        self.outtxt.append(f"  INITIAL CROSS-SECTION")
        self.outtxt.append(f"    EFFECTIVE DEPTH ={DI:10.4f}  m")
        self.outtxt.append(f"    TENSILE STEEL   ={ASTLI:10.6f}  m**2")

        if ITCONF > 0:
            DC = -AAC / ASTLC + TC / 2.0
            self.outtxt.append(f"")
            self.outtxt.append(f"")
            self.outtxt.append(f"  CONFINED CROSS-SECTION")
            self.outtxt.append(f"    EFFECTIVE DEPTH ={DC:10.4f}  m")
            self.outtxt.append(f"    TENSILE STEEL   ={ASTLC:10.6f}  m**2")

        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f"")
        self.outtxt.append(f"*** COLUMN  INTERACTION  VALUES ***")
        self.outtxt.append(f"")
        self.outtxt.append(f" Uniaxial compression")
        self.outtxt.append(f"")
        self.outtxt.append(f"  M      CONCRETE          LOAD")
        self.outtxt.append(f"          STRAIN")
        self.outtxt.append(f"          ( m/m )         ( MN )")
        self.outtxt.append(f"")

        if ITCONF == 1:
            J = NSTRNC
        else:
            J = NSTRNI
        XXPMAX = 0.0
        PCA = np.zeros(len(STRN) + 3)
        XPA = np.zeros(len(STRN) + 3)
        PCC = np.zeros(len(STRN) + 3)
        XPC = np.zeros(len(STRN) + 3)

        for i in range(1, J + 1):
            IND = 0
            if i > NSTRNI:
                IND = 1

            XPA[i], PCA[i], XPC[i], PCC[i] = PLCENT(STRN[i], EC, FC, ECS, FCS, FYS, YMS, ESH, YSH, FSU, ESU,
                                                    C1, C2, IND, ISPALL, AG, AST, AAS, AGC, TYPEC, ITCONF,
                                                    A4I, A4C, A4S, A4SS, RECS)

            if i > 0:
                if IND != 1:
                    self.outtxt.append(
                        f"{i:3}{STRN[i]:15.5f}{XPA[i]:15.3f}   INITIAL CROSS-SECTION")
                    if abs(PCA[i]) > 0.00001:
                        self.outtxt.append(
                            '+' + ' ' * 50 + f"PL.CENTROID ={PCA[i]:10.4f}m")
                    if XPA[i] > XXPMAX:
                        XXPMAX = XPA[i]

                if ITCONF > 0:
                    self.outtxt.append(
                        f"{i:3}{STRN[i]:15.5f}{XPC[i]:15.3f}   CONFINED CROSS-SECTION")
                    if XPC[i] > XXPMAX:
                        XXPMAX = XPC[i]

        KJ = NSTRNI + 1
        XPA[KJ] = - FYS * (AST + AAS)
        E1 = XPA[KJ] * FSU / FYS
        E2 = -ESY
        E4 = -ESU

        self.outtxt.append(f"")
        self.outtxt.append(f"")

        KL = J + 1
        self.outtxt.append(
            f"{KL:3}{E2:14.5f}{XPA[KJ]:14.3f}   INITIAL CROSS-SECTION")

        if ITCONF > 0:
            JK = NSTRNC + 1
            XPC[JK] = -FYS * AST
            E3 = -FSU * AST
            self.outtxt.append(
                f"{KL:3}{E2:14.5f}{XPC[JK]:14.3f}   CONFINED CROSS-SECTION")

        KL = KL + 1
        self.outtxt.append(
            f"{KL:3}{E4:14.5f}{E1:14.3f}   INITIAL CROSS-SECTION")

        if ITCONF > 0:
            self.outtxt.append(
                f"{KL:3}{E4:14.5f}{E3:14.3f}   CONFINED CROSS-SECTION")

        self.outtxt.append("")
        self.outtxt.append("")
        self.outtxt.append("")
        self.outtxt.append(" Uniaxial Bending")
        self.outtxt.append("")
        self.outtxt.append(
            "        εc        N        M   M/My    1/r    Yield    εs       x     V     M(V)")
        self.outtxt.append(
            "      (m/m)     (MN)    (MN.m)       (rad/m)  Ductl.  (m/m)    (m)   (MN)  (MN.m)")

        XXPMIN = XPA[KJ]

        if NSTRNI >= 5:
            XPAA = XPA[5]
        else:
            XPAA = XPA[NSTRNI]

        if len(axial_loads) == 0:
            XPCALC = XPAA * default_norm_axials
            XPCALC = np.append(XPCALC, [XPA[KJ] * 0.3, XPA[KJ] * 0.6, XPA[KJ]])
        else:
            for i in range(0, NLOAD):
                XPCALC[i] = axial_loads[i]

        # goto.x9999

        K = -1

        label.x690  # **************************************************** 690
        K += 1
        # print('')
        # print('')
        # print(f'*******************************************************')
        # print(f'K = {K}    ({K + 1} in fortran)')
        # print(f'*******************************************************')
        # print('')
        XPU = XPCALC[K]
        ICONF = 0
        ERROR = max(abs(XPU / 200), 0.0045)
        ECLIM = ECULT + 0.001
        ESLIM = ESU + 0.001

        label.x700  # **************************************************** 700
        ICOUNT = 0

        label.x710  # **************************************************** 710
        ICOUNT += 1
        # print('')
        # print('---------------------------------')
        # print(f'ICOUNT = {ICOUNT}')
        # print('')
        C = 0.5 * T
        DPC = 0.0

        if ICOUNT < 4:
            goto.x740

        if ICONF == 1:
            goto.x720

        DPC = PCA[ICOUNT - 3]

        if XPU < XPA[ICOUNT - 3]:
            goto.x730

        goto.x930

        label.x720  # **************************************************** 720
        DPC = PCC[ICOUNT - 3]
        if XPU < XPC[ICOUNT - 3]:
            goto.x730
        goto.x930

        label.x730  # **************************************************** 730
        ECU = STRN[ICOUNT - 3]
        if ICOUNT > 4 and CCC[ICOUNT - 1] != 0.:
            C = CCC[ICOUNT - 1]

        label.x740  # **************************************************** 740
        KCOUNT = 0
        T1 = 0.0
        TL = T

        if ICONF == 1:
            TL = TC
        if ICOUNT == 1 or ICOUNT == 3:
            goto.x750
        if ICOUNT == 2:
            T1 = 0.5 * TL - YS[0]
        T3 = 8. * T
        goto.x760

        label.x750  # **************************************************** 750
        T1 = 0.0
        T3 = 0.5 * TL - YS[NBOTL - 1]
        if ICOUNT == 3:
            T3 = T3 * ECULT / (ESU + ECULT)
        if T3 <= C:
            C = T3 / 2.0

        label.x760  # **************************************************** 760
        KCOUNT += 1
        if KCOUNT == NITT:
            goto.x930

        if ICOUNT == 1:
            ECU = C * ESY / (TL / 2. - YS[NBOTL - 1] - C)
        elif ICOUNT == 2:
            ECU = - C * ESY / (TL / 2. - YS[0] - C)
        elif ICOUNT == 3:
            ECU = C * ESU / (TL / 2. - YS[NBOTL - 1] - C)

        # print(f'K={K:2}  ICOUNT={ICOUNT:2}   KCOUNT={KCOUNT:2}    ECU={ECU:10.6f}')

        if ECU > ECLIM:
            goto.x930

        XP = 0.0
        XM = 0.0
        ASTL = 0.0
        XX = 0.0

        if ECU < 0.0:
            goto.x840

        if TYPEC == 1:
            goto.x770
        elif TYPEC == 2:
            goto.x800
        elif TYPEC == 3:
            goto.x820

        label.x770  # **************************************************** 770
        if ICONF == 1:
            goto.x780
        XP, XM = CFRCR(C, ECU, T, B, EC, FC)
        goto.x840

        label.x780  # **************************************************** 780
        if TYPES == 1:
            goto.x790
        else:
            goto.x810

        label.x790  # **************************************************** 790
        XP, XM = CFRCR(C, ECU, TC, BC, EC, FC)
        goto.x840

        label.x800  # **************************************************** 800
        if ICONF == 1:
            goto.x810
        XP, XM = CFRCS(C, T, ECU, A, Y, EC, FC)
        goto.x840

        label.x810  # **************************************************** 810
        XP, XM = CFRCS(C, TC, ECU, AC, YC, EC, FC)
        goto.x840

        label.x820  # **************************************************** 820
        if ICONF == 1:
            goto.x830
        XP, XM = CFRCS(C, T, ECU, A, Y, EC, FC)
        # print(f'{XP}   {XM}')
        if ISPALL == 0:
            goto.x840
        XPS = 0.0
        XMS = 0.0
        XPS, XMS = CFRCS(C, T, ECU, AC, YC, ECS, FCS)
        XP = XP + XPS
        XM = XM + XMS
        goto.x840

        label.x830  # **************************************************** 830
        XP, XM = CFRCS(C, TC, ECU, AC, YC, EC, FC)

        label.x840  # **************************************************** 840
        for N in range(0, NL):
            ES[N] = (C - TL / 2. + YS[N]) * ECU / C
            FS = STRHRD(ES[N], FYS, YMS, ESH, YSH, FSU, ESU, C1, C2)
            if ES[N] < 0.0:
                goto.x850
            FCI = STRESS(ES[N], EC, FC)
            FS = FS - FCI

            label.x850  # **************************************************** 850
            XP = XP + AS[N] * FS

            label.x860  # **************************************************** 860
            XM = XM + AS[N] * FS * YS[N]

        if ISSTL == 0 or ICONF == 1:
            goto.x910

        for N in range(0, ISSTL):
            ES[N] = (C - TL / 2. + YSS[N]) * ECU / C
            FS = STRHRD(ES[N], FYS, YMS, ESH, YSH, FSU, ESU, C1, C2)
            if ES[N] < 0.:
                goto.x890
            if ISPALL == 0:
                goto.x870
            FCI = STRESS(ES[N], ECS, FCS)
            goto.x880

        label.x870  # **************************************************** 870
        FCI = STRESS(ES[N], EC, FC)

        label.x880  # **************************************************** 880
        if FCI >= 0.:
            goto.x900
        FS = FS - FCI

        label.x890  # **************************************************** 890
        XP = XP + ASS[N] * FS
        XM = XM + ASS[N] * FS * YSS[N]

        label.x900  # **************************************************** 900
        pass

        label.x910  # **************************************************** 910
        pass

        XP = -XP
        XM = -XM
        if ES[NBOTL] < -ESLIM or ES[0] > ESLIM:
            goto.x930
        if abs(XPU - XP) < ERROR:
            goto.x950
        if XP < XPU:
            goto.x920
        T3 = C
        C = (C + T1) / 2.0
        goto.x760

        label.x920  # **************************************************** 920
        T1 = C
        C = (C + T3) / 2.0
        goto.x760

        label.x930  # **************************************************** 930
        CCC[ICOUNT] = 0.0
        EES[ICOUNT] = 0.0
        ECC[ICOUNT] = 0.0
        DUCT1[ICOUNT] = 0.0
        O[ICOUNT] = 0.0
        if ICONF == 1:
            goto.x940

        XXMI[K, ICOUNT] = 0.0
        PHII[K, ICOUNT] = 0.0
        VI[ICOUNT] = 0.0
        VMI[K, ICOUNT] = 0.0

        goto.x970

        label.x940  # **************************************************** 940
        XXMC[K, ICOUNT] = 0.0
        PHIC[K, ICOUNT] = 0.0
        VC[ICOUNT] = 0.0
        VMC[K, ICOUNT] = 0.0
        goto.x1000

        label.x950  # **************************************************** 950
        if ICONF == 1:
            goto.x980

        if TYPEC != 3:
            goto.x960

        if ICOUNT >= 4:
            goto.x960

        XXP, DPC, E1, E2 = PLCENT(ECU, EC, FC, ECS, FCS, FYS, YMS, ESH, YSH, FSU, ESU, C1, C2,
                                  0, ISPALL, AG, AST, AAS, AGC, TYPEC, ITCONF, A4I, A4C, A4S, A4SS, RECS)

        print(f'{XXP} {DPC} {E1} {E2}')

        label.x960  # **************************************************** 960
        XXMI[K, ICOUNT] = XM - XP * DPC
        CCC[ICOUNT] = C
        ECC[ICOUNT] = ECU
        EES[ICOUNT] = ES[NBOTL - 1]
        PHII[K, ICOUNT] = abs(ECU / C)
        DUCT1[ICOUNT] = 0.0
        O[ICOUNT] = 0.0
        DUCT1, O = YLDDUC(PHII, PHII, XXMI, XXMI, ICOUNT, K, DUCT1, O)

        if S != 0.0:
            VI, VMI = SHEAR(VI, VMI, ICOUNT, K, AG, G, TL, AST,
                            XL, XP, F, ASTLI, DI, B, TYPEC, TYPES, ICONF)

        label.x970  # **************************************************** 970
        if ICOUNT < JJ:
            goto.x710

        DUCTI, AREA, DUCMXI = DUCTLE(
            PHII, XXMI, AREA, DUCTI, DUCMXI, K, 1, NSTRNI, NSTRNC)
        self.outtxt.append('')
        self.outtxt.append('')

        _data = []
        for i in range(1, 4):
            self.outtxt.append((f"{K + 1:2}" if i == 1 else '  ') +
                               f" {abc[i]}{ECC[i]:8.5f}{XPU:9.4f}{XXMI[K, i]:9.4f}{O[i]:5.2f}{PHII[K, i]:10.6f}" +
                               f"{DUCT1[i]:6.2f}{EES[i]:9.5f}{CCC[i]:7.3f}{VI[i]:7.3f}{VMI[K, i]:7.3f}")
            _data_row = {'K': K + 1,
                         'i': abc[i],
                         'εc': ECC[i],
                         'N': XPU,
                         'M': XXMI[K, i],
                         'M/My':O[i],
                         'φ': PHII[K, i],
                         'Duct': DUCT1[i],
                         'εs': EES[i],
                         'x': CCC[i],
                         'V': VI[i],
                         'M(V)': VMI[K, i]}
            _data.append(_data_row)

        III = 3
        IREM = 0

        for i in range(4, JJ + 1):
            if ECC[i] > 0:
                self.outtxt.append(
                    f"{i - 3:4}{ECC[i]:8.5f}{XPU:9.4f}{XXMI[K, i]:9.4f}{O[i]:5.2f}{PHII[K, i]:10.6f}" +
                    f"{DUCT1[i]:6.2f}{EES[i]:9.5f}{CCC[i]:7.3f}{VI[i]:7.3f}{VMI[K, i]:7.3f}")

                _data_row = {'K': K + 1,
                            'i': i - 3,
                            'εc': ECC[i],
                            'N': XPU,
                            'M': XXMI[K, i],
                            'M/My':O[i],
                            'φ': PHII[K, i],
                            'Duct': DUCT1[i],
                            'εs': EES[i],
                            'x': CCC[i],
                            'V': VI[i],
                            'M(V)': VMI[K, i]}
                _data.append(_data_row)
        
        self.uniaxial_bending_ini_dfs.append(pd.DataFrame(data=_data))

        label.x975  # **************************************************** 975
        if ITCONF == 0:
            goto.x1010
        ICONF = 1
        goto.x700

        label.x980  # **************************************************** 980
        if TYPEC != 3:
            goto.x990
        if ICOUNT >= 4:
            goto.x990

        E1, E2, XXP, DPC = PLCENT(ECU, EC, FC, ECS, FCS, FYS, YMS, ESH, YSH, FSU, ESU, C1, C2,
                                  0, ISPALL, AG, AST, AAS, AGC, TYPEC, ITCONF, A4I, A4C, A4S, A4SS, RECS)

        label.x990  # **************************************************** 990
        XXMC[K, ICOUNT] = XM - XP * DPC
        CCC[ICOUNT] = C
        EES[ICOUNT] = ES[NBOTL]
        ECC[ICOUNT] = ECU
        PHIC[K, ICOUNT] = abs(ECU / C)
        DUCT1[ICOUNT] = 0.0
        O[ICOUNT] = 0.0
        DUCT1, O = YLDDUC(PHIC, PHII, XXMC, XXMI, ICOUNT, K, DUCT1, O)

        if S != 0.0:
            VC, VMC = SHEAR(VC, VMC, ICOUNT, K, AGC, G, TL, AST,
                            XL, XP, F, ASTLC, DC, BC, TYPEC, TYPES, ICONF)

        label.x1000  # **************************************************** 1000
        if ICOUNT < JJJ:
            goto.x710
        DUCTC, AREA, DUCMXC = DUCTLE(
            PHIC, XXMC, AREA, DUCTC, DUCMXC, K, 2, NSTRNI, NSTRNC)

        self.outtxt.append('')
        self.outtxt.append('')
        _data = []
        for i in range(1, 4):
            self.outtxt.append((f"{K + 1:2}" if i == 1 else '  ') +
                               f" {abc[i]}{ECC[i]:8.5f}{XPU:9.4f}{XXMC[K, i]:9.4f}{O[i]:5.2f}{PHIC[K, i]:10.6f}" +
                               f"{DUCT1[i]:6.2f}{EES[i]:9.5f}{CCC[i]:7.3f}{VC[i]:7.3f}{VMC[K, i]:7.3f}")

            _data_row = {'K': K + 1,
                         'i': abc[i],
                         'εc': ECC[i],
                         'N': XPU,
                         'M': XXMC[K, i],
                         'M/My':O[i],
                         'φ': PHIC[K, i],
                         'Duct': DUCT1[i],
                         'εs': EES[i],
                         'x': CCC[i],
                         'V': VC[i],
                         'M(V)': VMC[K, i]}
            _data.append(_data_row)

        for i in range(4, JJJ + 1):
            if ECC[i] > 0:
                self.outtxt.append(f"{i - 3:4}{ECC[i]:8.5f}{XPU:9.4f}{XXMC[K, i]:9.4f}{O[i]:5.2f}{PHIC[K, i]:10.6f}" +
                                   f"{DUCT1[i]:6.2f}{EES[i]:9.5f}{CCC[i]:7.3f}{VC[i]:7.3f}{VMC[K, i]:7.3f}")
                
                _data_row = {'K': K + 1,
                        'i': i-3,
                        'εc': ECC[i],
                        'N': XPU,
                        'M': XXMC[K, i],
                        'M/My':O[i],
                        'φ': PHIC[K, i],
                        'Duct': DUCT1[i],
                        'εs': EES[i],
                        'x': CCC[i],
                        'V': VC[i],
                        'M(V)': VMC[K, i]}
                _data.append(_data_row)

        self.uniaxial_bending_conf_dfs.append(pd.DataFrame(data=_data))

        label.x1010  # **************************************************** 1010
        if K <= NLOAD - 2:
            goto.x690

        label.x9999
        pass



'''
Για να καλέσω τις συναρτήσεις γράφω στο UDF modules στο excel:
- streng.xwflib.xwf_main για να καλέσω τη συλολική
- streng.xwflib.(το όνομα του module) για να καλέσω μεμονωμένα
- ή κάνω ένα νέο .py αρχείο στον ίδο φάκελο με το αρχείο του excel και φωνάζω ότι θέλω εκεί
'''

# Γενικά imports
import pandas as pd
import xlwings as xw
import numpy as np

# xwf imports
from ..xwflib.math.xwf_math import *
from ..xwflib.codes.eurocodes.ec2_raw_ch3 import *
from ..xwflib.docs_examples import *
from ..xwflib.ppp.loads import *


from ..tools.bilin import Bilin



@xw.func
@xw.arg('x', np.array, ndim=1)
@xw.arg('y', np.array, ndim=1)
@xw.ret(index=False, expand='table')
def xwf_bilin(x, y, x_target):
    bl = Bilin(xtarget=x_target)
    bl.curve_ini.x = x
    bl.curve_ini.y = y
    bl.calc()

    _res = bl.bilinear_curve.all_quantities.to_panda_dataframe

    return _res




import math
import numpy as np
# from scipy import interpolate

# Στοιχεία φατνώματος
l = 4.0           # [m] Το άνοιγμα του πλαισίου
h = 3.0           # [m] Το ύψος τους ορόφου



# Στοιχεία τοιχοποιίας
k = 0.4         # εμπειρικός συντελεστής που λαμβάνει υπόψη την ομάδα στην οποία κατατάσσονται τα
                # τοιχωσώματα και το είδος του κονιάματος δόμησης, παίρνει δε τιμές από 0,35 έως 0,55.
fbc = 4.0       # [MPa] η θλιπτική αντοχή του τοιχοσώματος (ενδεικτικές τιμές για υφιστάμενες τοιχοποιίες 3.1 -5.5 MPa)
fmc = 1.6       # [MPa] η θλιπτική αντοχή του κονιάματος (ενδεικτικές τιμές για υφιστάμενες τοιχοποιίες 1.5 -1.8  MPa)

t = 0.20        # Το πάχος t της θλιβόμενης διαγωνίου ή το πάχος του τοίχου;

# Συντελεστές
λΕ = 500.0     # Ο συντελεστής που θα μπεί στη σχέση Ew = λΕ * fcws. Παίρνει τιμές 500 - 1000
λγy = 1.25      # Ο συντελεστής που θα μπεί στη σχέση γy = (l/h+h/l)*(1-1.5)*10^-3. Τιμές από 1 - 1.5. Για το γu με παρεμβολή

# Γεωμετρικά στοιχεία
α = np.arctan(h/l)  # Η γωνία με την οριζόντιο
L = l / np.cos(α)   # Το καθαρό μήκος της διαγωνίου
λ = L / t           # Η λυγηρότητα της τοιχοπλήρωσης

print('l = {0:.2f}m'.format(l))
print('h = {0:.2f}m'.format(h))

print('α = {0:.3f}rad'.format(α))
print('L = {0:.2f}m'.format(L))
print('λ = {0:.2f}'.format(λ))


# Αντοχή της τοιχοπλήρωσης
fcws = 1.25 * k * math.pow(fbc, 0.7) * math.pow(fmc, 0.3)
Ew = λΕ * fcws    # 500 - 1000

if λ < 15:
    φ = 1.0
elif λ>30:
    φ =  0.0
else:
    φ500 = 0.9 * 1 / math.exp(math.pow((0.0447 * λ - 0.063), 2) / 0.9)
    φ1000 = 0.9 * 1 / math.exp(math.pow((0.0316 * λ - 0.063), 2) / 0.9)
    x_φ = [500, 1000]
    y_φ = [φ500, φ1000]
    φ = np.interp(λΕ, x_φ, y_φ)

print('fcws = {0:.3f}MPa'.format(fcws))
print('Ew = {0:.1f}MPa'.format(Ew))
#print('φ500 = {0:.3f}'.format(φ500))
#print('φ1000 = {0:.3f}'.format(φ1000))
print('φ = {0:.3f}'.format(φ))


# Διάγραμμα τ - γ
γy = (l / h + h / l) * λγy * 0.001
λγu = np.interp(λγy, [1.0, 1.5], [2.0, 3.5])
γu = (l / h + h / l) * λγu * 0.001

print('γy = {0:.5f}'.format(γy))
print('γu = {0:.5f}'.format(γu))




b = 0.15 * L






N = 1000.0 * t * b * fcws     # *1000 για να γίνει kNs
V = N * math.cos(α)














print('b = {0:.2f}m'.format(b))




print('N = {0:.3f}kN'.format(N))
print('V = {0:.3f}kN'.format(V))










# import math


def θycalc(φy, Ls, av, z, h, db, fy, fc, units='m-MPa'):
    """ Στροφή διαρροής

    .. highlight:: python
    .. code-block:: python
       :linenos:

        theta = rotation.θy(φy=0.00638,
                            Ls=2.525,
                            av=1,
                            z=0.506,
                            h=0.506,
                            db=0.018,
                            fy=500.,
                            fc=16,
                            units='m-MPa')

    Args:
        φy (float): Καμπυλότητα διαρροής [m-1]
        Ls (float): Μήκος διάτμησης [m]
        av (float): συντελεστής που ισούται με 1 εάν η τέμνουσα VRc που προκαλεί λοξή ρηγμάτωση του στοιχείου,
            υπολείπεται της τιμής της τέμνουσας κατά την καμπτική διαρροή VMu=My/Ls, και με 0 αν είναι μεγαλύτερη
        z (float): Ο μοχλοβραχίονας των εσωτερικών δυνάμεων
        h (float): το ύψος της διατομής [m]
        db (float): η διάμετρος των διαμήκων ράβδων [m]
        fy (float): το όριο διαρροής του χάλυβα [MPa]
        fc (float): η θλιπτική αντοχή του σκυροδέματος [MPa]

    Returns:
        float: Given using the expression:

        .. math:: θ_y=φ_y\dfrac{L_s+a_vz}{3}+0.0014 \Big(1+1.5\dfrac{h}{L_s} \Big)+\dfrac{φ_yd_bf_y}{8\sqrt{f_c}}
    """
    return φy * (Ls + av * z) / 3 + 0.0014 * (1 + 1.5 * h / Ls) + φy * db * fy / (8 * fc**0.5)


def θum(ν, ωtot, ω2, αs, α, ρs, ρd, fc, fyw, units='MPa'):
    """ Στροφή αστοχίας θum

     .. highlight:: python
     .. code-block:: python
        :linenos:

        theta_u = rotation.θum(ν=0.093,
                               ωtot=0.351,
                               ω2=0.132,
                               αs=3.78,
                               α=0.4737,
                               ρs=0.00279,
                               ρd=0.,
                               fc=16.,
                               fyw=500.,
                               units='MPa')

     Args:
         ν (float): ανηγμένο αξονικό
         ωtot (float): Συνολικό μηχανικό ποσοστό οπλισμού
         ω2 (float): Μηχανικό ποσοστό θλιβόμενου οπλισμού
         αs (float): Λόγος διάτμησης
         ρs (float): Γεωμετρικό ποσοστό εγκάρσιου οπλισμού
         ρd (float): Γεωμετρικό ποσοστό δισδιαγώνιου οπλισμού
         fc (float): η θλιπτική αντοχή του σκυροδέματος [MPa]
         fyw (float): το όριο διαρροής του χάλυβα ου εγκάρσιου οπλισμού [MPa]

     Returns:
         float: Given using the expression:

         .. math:: θ_{um}=0.016\cdot(0.3^ν)\cdot\Bigg[\dfrac{max(0.01,ω')}{max(0.01, (ω_{tot}-ω'))} \cdot f_c\Bigg]^{0.225}\cdot (α_s)^{0.35} \cdot 25^{(α\cdot ρ_s \cdot \dfrac{f_{yw}}{f_c})}\cdot (1.25^{100ρ_d})

     """
    part1 = (fc * max(0.01, ω2) / max(0.01, (ωtot-ω2)))**0.225
    part2 = αs**0.35
    part3 = 25.0**(α * ρs * fyw / fc)
    part4 = 1.25**(100 * ρd)
    return 0.016 * 0.3**ν * part1 * part2 * part3 * part4


def αcalc(sh, bc, hc, Σbi2):
    α = (1-sh/(2*bc))*(1-sh/(2*hc))*(1-Σbi2/(6*bc*hc))
    return max(0.0, α)


from dataclasses import dataclass
from openseespy import opensees

@dataclass
class ModifiedIbarraKrawinkler:
    n: float  # Usual value = 10

    E: float  # Modulus of elasticity
    I: float  # Moment of inertia
    L: float  # Element length
    # Positive moments data
    My_P: float  # yield moment
    McMy_P: float  # ratio of capping moment to yield moment, Mc / My
    th_pP: float  # plastic rot capacity
    th_pcP: float  # post-capping rot capacity
    th_uP: float  # ultimate rot capacity
    # Negative moments data
    My_N: float
    McMy_N: float
    th_pN: float
    th_pcN: float
    th_uN: float

    LS: float = 1000.0  # basic strength deterioration (a very large # = no cyclic deterioration)
    LK: float = 1000.0  # unloading stiffness deterioration (a very large # = no cyclic deterioration)
    LA: float = 1000.0  # accelerated reloading stiffness deterioration (a very large # = no cyclic deterioration)
    LD: float = 1000.0  # post-capping strength deterioration (a very large # = no deterioration)
    cS: float = 1.0  # exponent for basic strength deterioration (c = 1.0 for no deterioration)
    cK: float = 1.0  # exponent for unloading stiffness deterioration (c = 1.0 for no deterioration)
    cA: float = 1.0  # exponent for accelerated reloading stiffness deterioration (c = 1.0 for no deterioration)
    cD: float = 1.0  # exponent for post-capping strength deterioration (c = 1.0 for no deterioration)
    ResP: float = 0.4  # residual strength ratio for pos loading
    ResN: float = 0.4  # residual strength ratio for neg loading
    DP: float = 1.0  # rate of cyclic deterioration for pos loading (this parameter is used to create assymetric hysteretic behavior for the case of a composite beam). For symmetric hysteretic response use 1.0
    DN: float = 1.0  # rate of cyclic deterioration for neg loading (this parameter is used to create assymetric hysteretic behavior for the case of a composite beam). For symmetric hysteretic response use 1.0

    def __post_init__(self):
        self.calc()

    def calc(self):
        self.Imod = self.I * (self.n + 1) / self.n
        self.K = self.n * 6 * self.E * self.Imod / self.L

        # strain hardening ratio of spring
        self.a_memPos = (self.n + 1.) * self.My_P * (self.McMy_P - 1.0) / (self.K * self.th_pP)
        self.a_memNeg = (self.n + 1.) * abs(self.My_N) * (self.McMy_N - 1.0) / (self.K * self.th_pN)

        # modified strain hardening ratio of spring (Ibarra & Krawinkler 2005, note: Eqn B.5 is incorrect)
        self.asPos = self.a_memPos / (1. + self.n * (1. - self.a_memPos))
        self.asNeg = self.a_memNeg / (1. + self.n * (1. - self.a_memNeg))

    def opensees_bilin_py(self, eleID, nodeStart, nodeEnd, change_order=False):
        opensees.uniaxialMaterial('Bilin', eleID, self.K, self.asPos, self.asNeg, self.My_P, self.My_N,
                                  self.LS, self.LK, self.LA, self.LD, self.cS, self.cK, self.cA, self.cD,
                                  self.th_pP, self.th_pN, self.th_pcP, self.th_pcN,
                                  self.ResP, self.ResN, self.th_uP, self.th_uN, self.DP, self.DN)

        #         print(f'{eleID}, {self.K}, {self.asPos}, {self.asNeg}, {self.My_P}, {self.My_N},'+
        #                 f'{self.LS}, {self.LK}, {self.LA}, {self.LD}, {self.cS}, {self.cK}, {self.cA}, {self.cD},'+
        #                 f'{self.th_pP}, {self.th_pN}, {self.th_pcP}, {self.th_pcN},'+
        #                 f'{self.ResP}, {self.ResN}, {self.th_uP}, {self.th_uN}, {self.DP}, {self.DN}')

        opensees.element('zeroLength', eleID, nodeStart, nodeEnd, '-mat', eleID, '-dir', 6)
        if change_order == False:
            opensees.equalDOF(nodeStart, nodeEnd, 1, 2)
        else:
            opensees.equalDOF(nodeEnd, nodeStart, 1, 2)

    def opensees_peak_py(self, eleID, nodeStart, nodeEnd, change_order=False):
        opensees.uniaxialMaterial('ModIMKPeakOriented', eleID, self.K, self.asPos, self.asNeg, self.My_P, self.My_N,
                                  self.LS, self.LK, self.LA, self.LD, self.cS, self.cK, self.cA, self.cD,
                                  self.th_pP, self.th_pN, self.th_pcP, self.th_pcN,
                                  self.ResP, self.ResN, self.th_uP, self.th_uN, self.DP, self.DN)

        #         print(f'{eleID}, {self.K}, {self.asPos}, {self.asNeg}, {self.My_P}, {self.My_N},'+
        #                 f'{self.LS}, {self.LK}, {self.LA}, {self.LD}, {self.cS}, {self.cK}, {self.cA}, {self.cD},'+
        #                 f'{self.th_pP}, {self.th_pN}, {self.th_pcP}, {self.th_pcN},'+
        #                 f'{self.ResP}, {self.ResN}, {self.th_uP}, {self.th_uN}, {self.DP}, {self.DN}')

        opensees.element('zeroLength', eleID, nodeStart, nodeEnd, '-mat', eleID, '-dir', 6)
        if change_order == False:
            opensees.equalDOF(nodeStart, nodeEnd, 1, 2)
        else:
            opensees.equalDOF(nodeEnd, nodeStart, 1, 2)

    def opensees_command_tcl(self, eleID, nodeR, nodeC):
        return f'rotSpring2DModIKModel {eleID:7} {nodeR:5} {nodeC:7} ' \
            f'{self.K:11.1f} {self.asPos:12.4e} {self.asNeg:12.4e} {self.My_P:10.3f} {self.My_N:10.3f} ' \
            f'{self.LS:9.2f} {self.LK:9.2f} {self.LA:9.2f} {self.LD:9.2f} ' \
            f'{self.cS:6.3f} {self.cK:6.3f} {self.cA:6.3f} {self.cD:6.3f} ' \
            f'{self.th_pP:10.3e} {self.th_pN:10.3e} {self.th_pcP:10.3e} {self.th_pcN:10.3e} ' \
            f'{self.ResP:6.3f} {self.ResN:6.3f} {self.th_uP:10.3e} {self.th_uN:10.3e} {self.DP:6.3f} {self.DN:6.3f};'



'''
Classes for managing output results
'''

import pandas as pd
from dataclasses import dataclass, field
from typing import List, Dict
from tabulate import tabulate
from .lists_and_dicts_and_pandas_etc import *


@dataclass
class OutputString:
    """ An output given as a list of strings

        .. uml::

            class OutputString {
            .. members ..
            + data: List[str] = field(default_factory=list)
            }


    Attributes:
        data (List[str]): A list of strings

    """
    data: List[str] = field(default_factory=list)
    # name: str = None

    def __str__(self):
        return '\n'.join(self.data)


@dataclass
class OutputTable:
    """ An output table given as a list of dictionaries.

        .. uml::

            class OutputTable {
            .. members ..
            + data: List[dict] = field(default_factory=list)
            .. properties ..
            + to_markdown()
            + to_panda_dataframe()
            .. methods ..
            + retrieve()
            + retrieve_column_to_list()
            + to_quantity_value()
            }

    It can be converted for presentation (or usage) as a pandas dataframe or a markdown table

    Attributes:
        data (List[dict]): A list of dictionaries

    """
    data: List[dict] = field(default_factory=list)


    @property
    def to_markdown(self):
        """ Converts data (list of dicts) to a markdown table using tabulate.

        Returns:
            str: a markdown table
        """
        return convert_lod_to_md_table(self.data)

    @property
    def to_panda_dataframe(self):
        """ Converts data to a pandas dataframe

        Returns:
            pd.DataFrame: a pandas dataframe
        """
        return convert_lod_to_pddf(self.data)

    def retrieve(self, search_field, search_value, find_field):
        res = [d[find_field] for d in self.data if d[search_field] == search_value]
        return res[0]

    def retrieve_column_to_list(self, column_name):
        return [k[column_name] for k in self.data]

    def to_quantity_value(self, selected_row=0):
        out = OutputTable()
        out.data = convert_dict_to_quantity_value(self.data[selected_row])
        return out

    def __str__(self):
        return self.to_markdown


@dataclass
class OutputExtended:
    """ A class for multiple outputs

        .. uml::

            class OutputExtended {
            .. members ..
            + outputTables: Dict[str, OutputTable] = field(default_factory=dict)
            + outputStrings: Dict[str, OutputString] = field(default_factory=dict)
            .. properties ..
            + print_all_outputStrings()
            .. methods ..
            + save_tofile_all_outputStrings()
            }

    Attributes:
        outputTables (dict[str, OutputTable]): A dictionary of OutputTables with a string key
        outputStrings (dict[str, OutputString]): A dictionary of OutputStrings with a string key

    """
    outputTables: Dict[str, OutputTable] = field(default_factory=dict)
    outputStrings: Dict[str, OutputString] = field(default_factory=dict)

    @property
    def print_all_outputStrings(self):
        all = []

        for key, value in self.outputStrings.items():
            all.extend(value.data)

        return '\n'.join(all)

    def save_tofile_all_outputStrings(self, pathfilename):
        file = open(pathfilename, 'w', encoding='utf8')
        file.write(self.print_all_outputStrings)
        file.close()




import numpy as np
from dataclasses import dataclass, astuple, field


@dataclass
class FirstRow:
    TYPEC: int
    TYPES: int
    ISPALL: int
    ICSTR: int
    ICSTRS: int
    NPTS: int
    NPTSS: int
    ITCONF: int
    ISSTL: int
    IECU: int
    ISSTR: int
    IGRAPH: int


@dataclass
class SecondRow:
    NLOAD: int
    NLAYER: int
    NDATA1: int
    NDATA2: int
    NSTRNI: int
    NSTRNC: int
    NITT: int


@dataclass
class ConcSectInfo:
    T: float
    B: float
    TC: float
    BC: float


@dataclass
class ReinfPlaceInfoTYPES12:
    GT: float
    NB1: int
    AS1: float
    NB2: int
    AS2: float
    NBAR: int
    AF: float


@dataclass
class ReinfPlaceInfoTYPES3:
    NL: int
    NBAR: int
    YS: list
    AS: list
    NBS: list
    IBUC: list
    YSS: list
    ASS: list
    NBSS: list


@dataclass
class LongReinfProps:
    FYS: float
    YMS: float
    ESH: float
    YSH: float
    ESU: float
    FSU: float
    ESDYN: float  # For ISSTR=1


@dataclass
class TransReinfInfo:
    FYSTRP: float
    ASSTRP: float
    S: float
    XL: float
    XSTRP: float
    ESUSTRP: float


@dataclass
class ConcPropsICSTR2348:
    FCJ: float
    S: float
    PPP: float


@dataclass
class ConcUnconfinedPropsICSTRS2:
    FCJS: float
    SS: float
    PPX: float
    NSP: int


@dataclass
class GeometryConcCrossSectionInfo:
    AA: list
    N1: list
    A1: list
    AAc: list
    N1c: list


@dataclass
class RccolaInput:
    first_row: FirstRow
    second_row: SecondRow
    conc_sect_info: ConcSectInfo
    reinf_place_info_TYPES12: ReinfPlaceInfoTYPES12
    reinf_place_info_TYPES3: ReinfPlaceInfoTYPES3
    long_reinf_props: LongReinfProps
    trans_reinf_info: TransReinfInfo
    conc_props_ICSTR2348: ConcPropsICSTR2348
    conc_unconfined_props_ICSTRS2: ConcUnconfinedPropsICSTRS2
    geometry_conc_crossec_info: GeometryConcCrossSectionInfo
    axials: np.array
    strains: np.array = None
    strains_defaults: int = None

    def run(self):
        if self.strains_defaults == 1:
            self.strains = np.array([0.0, 0.001, 0.002, 0.003, 0.0035, 0.004, 0.005, 0.006, 0.007, 0.008,
                                     0.009, 0.01, 0.011, 0.012, 0.013, 0.014, 0.015, 0.016, 0.018, 0.02,
                                     0.022, 0.024, 0.026, 0.028, 0.03])
            self.second_row.NSTRNI = 4
            self.second_row.NSTRNC = 24




from dataclasses import dataclass


@dataclass
class IdaStep:
    base_record: str
    pga: float
    loss_index: float
    collapse: bool

    # ενδεχομένως να βάλω ως μεταβλητή την class με τα αποτελέσματα της ανάλυσης



import numpy as np
from dataclasses import dataclass, field
from typing import List
from ...math import numerical


@dataclass
class LineXY:
    x: np.array = None
    y: np.array = None
    name: str = None
    tags: List[str] = field(default_factory=list)

    def area_under_curve(self, x_start=None, x_end=None):
        if x_start is None:
            x_start = self.x[0]
        if x_end is None:
            x_end = self.x[-1]
        return numerical.area_under_curve(x=self.x,
                                          y=self.y,
                                          x_start=x_start,
                                          x_end=x_end)

    def xy_with_endpoints(self, x_start, x_end):
        return numerical.xy_with_endpoints(x=self.x,
                                           y=self.y,
                                           x_start=x_start,
                                           x_end=x_end)

    @classmethod
    def from_txt_delimited(cls, fname, delimiter=' ', xcol=0, ycol=1):
        _x, _y = np.loadtxt(fname, delimiter=delimiter, usecols=(xcol, ycol), unpack=True)
        return cls(x=_x, y=_y)

    def load_delimited(self, fname, delimiter=' '):
        self.x, self.y = np.loadtxt(fname, delimiter=delimiter, usecols=(0, 1), unpack=True)

    # not working...νομίζω
    def load_space_delimited_string(self, text, delimiter=' '):
        self.x_ini = np.array([], dtype=np.float)
        self.y_ini = np.array([], dtype=np.float)
        spl = text.splitlines()
        for ln in spl:
            xy = ln.split(delimiter)
            self.x_ini = np.append(self.x_ini, xy[0])
            self.y_ini = np.append(self.y_ini, xy[1])
        self.x_ini = self.x_ini.astype(float)
        self.y_ini = self.y_ini.astype(float)

    @property
    def x_max(self):
        return max(self.x)

    @property
    def x_min(self):
        return min(self.x)

    @property
    def y_max(self):
        return max(self.y)

    @property
    def y_min(self):
        return min(self.y)

    @property
    def y_max_index(self):
        return np.argmax(self.y)



from dataclasses import dataclass
from . import areas
from .combos import LongReinforcementLayer, TransReinforcementLayer

from streng.common.io import results_display_older


@dataclass
class RectangularSectionReinforcement:
    long1: LongReinforcementLayer
    long2: LongReinforcementLayer
    longV: LongReinforcementLayer
    disdia: LongReinforcementLayer
    trans: TransReinforcementLayer
    cnom: float
    side_hoop_legs: int = 0  # Πρόσθετα σκέλη συνδετήρα στη 2ουσα παρειά. Πχ Αν είναι 0 είναι 2τμητοι, αν είναι 2 είναι 4τμητοι

    @property
    def d1(self):
        return self.cnom + self.trans.dia + 0.5 * self.long1.dia_max

    @property
    def d2(self):
        return self.cnom + self.trans.dia + 0.5 * self.long2.dia_max

    @property
    def Astot(self):
        return self.long1.As + self.long2.As + self.longV.As





from streng.phd.building_models.model2d_older.etabs_input import EtabsModel2d

model = EtabsModel2d(input_excel_filename=r'4orofo.xlsm')
model.etabs_filename = 'testfile.e2k'

model.etabs_write_file()

print(model.show_tabulate_input('beam_loads'))
print(model.show_tabulate_input('node_loads'))





from dataclasses import dataclass, field
from openseespy import opensees
from streng.phd.building_models.model2d_older.model_input import Model2d

@dataclass
class OpenSeesModel2d(Model2d):
    # etabs_filename: str = ''

    # procedure to read
    def ElasticBeamColumn(self, eleTag, iNode, jNode, sectType, E, transfTag, M, massType):
        found = 0
        prop = WSection[sectType]

        A = prop[0]
        I = prop[1]
        opensees.element('elasticBeamColumn', eleTag, iNode, jNode, A, E, I, transfTag, '-mass', M, massType)

    def start(self):

        opensees.wipe()
        opensees.model('Basic', '-ndm', 2)

        #    units kN, m

        E = 29000000.0
        massX = 0.49
        M = 0.
        coordTransf = "Linear"  # Linear, PDelta, Corotational
        massType = "-lMass"  # -lMass, -cMass

        self.write_nodes()


    def write_nodes(self):
        ds = self.input_data['joint_coordinates']
        for i, row in ds.iterrows():
            _nodeID = int(row['NodeID'])
            _X = row['X']
            _Y = row['Z']
            opensees.node(_nodeID, _X, _Y)


mymodel = OpenSeesModel2d(input_excel_filename=r'D:\phd_closure\ch3\designs\eak_frames\9st_016\9eak016.xlsm')
mymodel.start()




"""
Concrete strength functions

    .. uml::

        class strength <<(M,#FF7700)>> {
        .. functions ..
        + fcd(acc, fck, γc)
        + fcm(fck)
        + fctk005(fck)
        + fctk095(fck)
        + fctm(fck)
        }

"""

import math


def fcd(acc, fck, γc):
    """Design value for the compressive strength of concrete

    Args:
        acc (float): Coefficient taking account of long term effects on the compressive strength
                     and of unfavourable effects resulting from the way the load is applied
        fck (float): Characteristic (5%) compressive strength of concrete
        γc (float): Safety factor

    Returns:
        float: Given using the expression:

        .. math::
            f_{cd}=a_{cc}\dfrac{f_{ck}}{γ_c}

    """
    return acc * fck / γc


def fcm(fck):
    """ Mean compressive strength at 28 days

    Args:
        fck (float): Characteristic (5%) compressive strength of concrete [MPa]

    Returns:
        float: Given using the expression:

        .. math::
            f_{cm}=f_{ck} + 8

    """
    return fck + 8


def fctm(fck):
    """ Mean tensile strength at 28 days

    Args:
        fck (float): Characteristic (5%) compressive strength of concrete [MPa]

    Returns:
        float: Given using the expression:

        .. math::

           f_{ctm} = 0.30\cdot f_{ck}^{2/3} for f_{ck}\le 50MPa

           f_{ctm} = 2.12\cdot ln(1+f_{cm}/10) for f_{ck}> 50MPa

    """
    if fck <= 50:
        _fctm = 0.3 * fck**(2.0 / 3.0)
#        _fctm = 0.3 * math.pow(fck, (2.0 / 3.0))
    else:
        _fctm = 2.12 * math.log(1 + (fcm(fck) / 10))

    return _fctm


def fctk005(fck):
    """ Characteristic (5% fractile) tensile strength of concrete [MPa]

    Args:
        fck (float): Characteristic (5%) compressive strength of concrete [MPa]

    Returns:
        float: Given using the expression:

        .. math::
            f_{ctk,0.05}=0.7\cdot f_{ctm}

    """
    return 0.7 * fctm(fck)


def fctk095(fck):
    """ Characteristic (95% fractile) tensile strength of concrete [MPa]

    Args:
        fck (float): Characteristic (5%) compressive strength of concrete [MPa]

    Returns:
        float: Given using the expression:

        .. math::
            f_{ctk,0.95}=1.3\cdot f_{ctm}

    """
    return 1.3 * fctm(fck)



# -*- coding: utf-8 -*-
#
# LICENSE
#
# Copyright © 2014-2017, GEM Foundation, Chiara Casotto, Anirudh Rao,
# Vitor Silva.
#
# The Risk Modeller's Toolkit is free software: you can redistribute
# it and/or modify it under the terms of the GNU Affero General Public
# License as published by the Free Software Foundation, either version
# 3 of the License, or (at your option) any later version.
#
# You should have received a copy of the GNU Affero General Public License
# along with OpenQuake. If not, see <http://www.gnu.org/licenses/>
#
# DISCLAIMER
# 
# The software Risk Modeller's Toolkit (rmtk) provided herein
# is released as a prototype implementation on behalf of
# scientists and engineers working within the GEM Foundation (Global
# Earthquake Model).
#
# It is distributed for the purpose of open collaboration and in the
# hope that it will be useful to the scientific, engineering, disaster
# risk and software design communities.
#
# The software is NOT distributed as part of GEM’s OpenQuake suite
# (http://www.globalquakemodel.org/openquake) and must be considered as a
# separate entity. The software provided herein is designed and implemented
# by scientific staff. It is not developed to the design standards, nor
# subject to same level of critical review by professional software
# developers, as GEM’s OpenQuake software suite.
#
# Feedback and contribution to the software is welcome, and can be
# directed to the risk scientific staff of the GEM Model Facility
# (risk@globalquakemodel.org).
#
# The Risk Modeller's Toolkit (rmtk) is therefore distributed WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# The GEM Foundation, and the authors of the software, assume no
# liability for use of the software.



import math
from dataclasses import dataclass
from streng.tools.rccola.rcinput import RccolaInput
from streng.ppp.sections.concrete.reinforcement.areas import As_layer, As


@dataclass
class Materials:
    conc_fc: float
    steel_fy: float
    steel_Es: float
    steel_εsh: float
    steel_Esh: float
    steel_εsu: float
    steel_fu: float
    steel_fyw: float
    steel_εsuw: float


@dataclass
class SquareColumn:
    b: float
    cover: float
    l: float
    dia_corner: float
    dia_side1: float
    dia_side2: float
    n_side1: int
    n_side2: int
    trans_pattern: int
    trans_dia: int
    trans_s: float
    materials: Materials
    rccola_input: RccolaInput

    def doit(self):
        b = self.b
        t = self.b
        cover = self.cover
        l = self.l
        dia_corner = self.dia_corner
        dia_side1 = self.dia_side1
        dia_side2 = self.dia_side2
        n_side1 = self.n_side1
        n_side2 = self.n_side2
        trans_pattern = self.trans_pattern
        trans_dia = self.trans_dia
        trans_s = self.trans_s

        tc = self.b - 2 * cover - trans_dia
        bc = tc
        gt = tc - 2 * trans_dia - dia_corner

        nb1 = 4 + 2 * (n_side1 + n_side2)
        as1 = (As_layer([4, 2 * n_side1, 2 * n_side2],
                        [dia_corner, dia_side1, dia_side2])) / nb1

        nb2 = nb1 - 4
        if nb2 > 0:
            as2 = (As_layer([2 * n_side1, 2 * n_side2], [dia_side1, dia_side2])) / nb2
        else:
            as2 = 0

        tempas_ac = As(trans_dia) / (tc ** 2 * trans_s)

        nbar = 0
        xstrp = 0.
        ppp = 0.

        if trans_pattern == 1:
            nbar = 0
            xstrp = 2
            ppp = (4 * tc) * tempas_ac
        elif trans_pattern == 2:
            nbar = 1
            xstrp = 2 + 2 ** 0.5
            ppp = (4 * tc + 4 * tc / 2 ** 0.5) * tempas_ac
        elif trans_pattern == 3:
            nbar = 2
            xstrp = 4
            ppp = (8 * tc + 4 * tc / 3.0) * tempas_ac
        elif trans_pattern == 4:
            nbar = 3
            xstrp = 4 + 2 ** 0.5
            ppp = (8 * tc + 4 * tc / 2.0 + 4 * tc / 2 ** 0.5) * tempas_ac
        elif trans_pattern == 5:
            nbar = 1
            xstrp = 3
            ppp = (6 * tc) * tempas_ac

        asstrp = xstrp * As(trans_dia)

        self.rccola_input.strains_defaults = 1

        self.rccola_input.first_row = RccolaInput.FirstRow(TYPEC=1,
                                                           TYPES=1,
                                                           ISPALL=0,
                                                           ICSTR=8,
                                                           ICSTRS=0,
                                                           NPTS=0,
                                                           NPTSS=0,
                                                           ITCONF=1,
                                                           ISSTL=0,
                                                           IECU=1,
                                                           ISSTR=0,
                                                           IGRAPH=3)

        self.rccola_input.second_row = RccolaInput.SecondRow(NLOAD=0,
                                                             NLAYER=0,
                                                             NDATA1=0,
                                                             NDATA2=0,
                                                             NSTRNI=4,
                                                             NSTRNC=24,
                                                             NITT=30)

        self.rccola_input.conc_sect_info = RccolaInput.ConcSectInfo(T=t,
                                                                    B=b,
                                                                    TC=tc,
                                                                    BC=bc)

        self.rccola_input.reinf_place_info_TYPES12 = RccolaInput.ReinfPlaceInfoTYPES12(GT=gt,
                                                                                       NB1=nb1,
                                                                                       AS1=as1,
                                                                                       NB2=nb2,
                                                                                       AS2=as2,
                                                                                       NBAR=nbar,
                                                                                       AF=None)

        self.rccola_input.long_reinf_props = RccolaInput.LongReinfProps(FYS=self.materials.steel_fy,
                                                                        YMS=self.materials.steel_Es,
                                                                        ESH=self.materials.steel_εsh,
                                                                        YSH=self.materials.steel_Esh,
                                                                        ESU=self.materials.steel_εsu,
                                                                        FSU=self.materials.steel_fu,
                                                                        ESDYN=None)

        self.rccola_input.trans_reinf_info = RccolaInput.TransReinfInfo(FYSTRP=self.materials.steel_fyw,
                                                                        ASSTRP=asstrp,
                                                                        S=trans_s,
                                                                        XL=l,
                                                                        XSTRP=xstrp,
                                                                        ESUSTRP=self.materials.steel_εsuw)

        self.rccola_input.conc_props_ICSTR2348 = RccolaInput.ConcPropsICSTR2348(FCJ=self.materials.conc_fc,
                                                                                S=trans_s,
                                                                                PPP=ppp)



# -*- coding: utf-8 -*-
"""
Created on Thu Oct  1 14:30:43 2015

@author: chiaracasotto
"""
import csv
from rmtk.vulnerability.common.utils import isNumber

def read_parameters(input_file):

    hysteresis = {}
    if input_file == 'Default':
        hysteresis['type'] = 'Default'
    else:
        rDisp,fForce,uForce = [],[],[]
        gK, gD, gF, gE = [],[],[],[]
        dmgType = []
        with open(input_file, 'rU') as f:
            data = csv.reader(f)
            for line in data:
                if line[0] == 'rDisp':
                    for values in line[1:]:
                        if isNumber(values): 
                            rDisp.append(float(values))
                if line[0] == 'fForce':
                    for values in line[1:]:
                        if isNumber(values): 
                            fForce.append(float(values))
                if line[0] == 'uForce':
                    for values in line[1:]:
                        if isNumber(values): 
                            uForce.append(float(values))
                if line[0][0:2] == 'gK':
                    temp = []
                    for values in line[1:]:
                        if isNumber(values):
                            temp.append(float(values))
                    gK.append(temp)
                if line[0][0:2] == 'gD':
                    temp = []
                    for values in line[1:]:
                        if isNumber(values):
                            temp.append(float(values))
                    gD.append(temp)
                if line[0][0:2] == 'gF':
                    temp = []
                    for values in line[1:]:
                        if isNumber(values):
                            temp.append(float(values))
                    gF.append(temp)
                if line[0] == 'gE':
                    for values in line[1:]:
                        if isNumber(values):
                            gE.append(float(values))
                if line[0] == 'dmgType':
                    for values in line[1:]:
                        if len(values) != 0:
                            dmgType.append(values)
        
        # Store all the data in the dictionary
        hysteresis = {'type': "user-defined", 'rDisp': None, 'fForce': None, 'uForce': None,
                    'gK': None, 'gD': None, 'gF': None, 'gE': None, 'dmgType': None}
        
        hysteresis['rDisp'] = rDisp
        hysteresis['fForce'] = fForce
        hysteresis['uForce'] = uForce
        hysteresis['gK'] = gK
        hysteresis['gD'] = gD
        hysteresis['gF'] = gF
        hysteresis['gE'] = gE
        hysteresis['dmgType'] = dmgType

    return hysteresis
            


import math
import numpy as np
from goto import with_goto

def init_arrays_no_length():
    EC = []
    FC = []
    ECS = []
    FCS = []

    return EC, FC, ECS, FCS


def BUCKL(AS1, AS2, AS, YS, NBS, IBUC, A1STRP, YSH, BC, TC, S, FSU, FYS, FYSTRP, ESH):
    outtxt = []

    ECBUC = 0.004

    NALARM = 0
    ASMIN = 0.

    FBUC = 16 * FYSTRP * A1STRP

    if len(AS) == 0:
        ASMIN = AS1
        FLBUC = AS1 * FYS
        if FLBUC > FBUC:
            NALARM += 1


    else:
        ASMIN = 1000.
        for i in range(len(AS)):
            if AS[i] != 0. and NBS[i] != 0. and IBUC[i] == 0:
                ASB = AS[i] / NBS[i]
                FLBUC = ASB * FYS
                if FLBUC > FBUC:
                    NALARM += 1
                if ASB < ASMIN:
                    ASMIN = ASB

    if NALARM >= 1:
        outtxt.append(f"*** ATTENTION : ***")
        outtxt.append(f" Tie yield strength is less than 1/16th the long. bar yield strength in:")
        outtxt.append(f"   ={NALARM:6.0f} cases -> Early buckling expected")

    DMIN = 2 * math.sqrt(ASMIN / math.pi)
    SL = 4 * S / DMIN

    if S != 0. and A1STRP != 0.:
        FK = 8 * A1STRP * YSH * (BC + TC) / (BC * TC * S)
        FHARD = FSU / FYS
        SLMAX = -6 + 40 * FHARD

        if SL <= SLMAX:

            if FHARD < 1.3:
                outtxt.append(f"*** WARNING : ***")
                outtxt.append(f" Hardening capacity of steel too low (fu/fy = {FHARD:5.2f} )")

            if SL < 7.0:
                outtxt.append(f"*** WARNING : ***")
                outtxt.append(f" Slenderness of longitudinal bars too low ( A ={SL:5.2f} < 7 )")

            ALOGK = math.log(FK, 10.)
            if ALOGK > 2.5:
                outtxt.append(f"*** WARNING : ***")
                outtxt.append(f" Stiffness of transverse bars too high ( log k ={ALOGK:5.2f} > 2.5 )")

            ZHTA = 0.17 * (SLMAX / 54) ** 0.75
            HTA = math.sqrt(345 / FYS)
            XSI = 1 - math.sqrt(FHARD / 1.5)
            FAC1 = 0.75 * (ALOGK - 2.5) * (SLMAX - SL) ** (1.5 * SL ** (-ZHTA))
            FAC2 = HTA * (1 - XSI * 1.5 ** (0.2 * SL - 2))
            FAC3 = 0.029 * SL * SL - 2.67 * SL + 64
            ECBUC = ESH + 0.001 * (FAC1 + FAC2 * FAC3)

        else:
            outtxt.append(f"*** ATTENTION : ***")
            outtxt.append(f" Slenderness of longitudinal bars too high -> Early buckling expected ! ")
            outtxt.append(f"")
            outtxt.append(f"      A = {SL:6.2f} > Amax = {SLMAX:6.2f}")

    else:
        outtxt.append(f"*** ATTENTION : NO TRANSVERSE REINFORCEMENT ! ***")

    return ECBUC, outtxt

@with_goto
def PLCENT(ECI, EC, FC, ECS, FCS, FYS, YMS, ESH, YSH, FSU, ESU, C1, C2, IND, ISPALL, AG, AST, AAS, AGC,
           TYPEC, ITCONF, A4I, A4C, A4S, A4SS, RECS):
    #
    #      CALCULATES PLASTIC CENTROID LOCATION (DPC) RELATIVE TO
    #      MID-DEPTH AND AXIAL LOAD (XXP) GIVEN CONCRETE STRAIN (EX)
    #

    XXP = 0.
    DPC = 0.
    XXPC = 0.
    DPCC = 0.

    FCI = - STRESS(ECI, EC, FC)
    FS = abs(STRHRD(ECI, FYS, YMS, ESH, YSH, FSU, ESU, C1, C2))

    if IND == 1:
        goto.x20
    if ISPALL == 0:
        goto.x10

    # SECTION WITH DIFFERENT CONCRETE IN COVER AND CORE
    FCJ = - STRESS(ECI, ECS, FCS)
    XXP = FCI * AGC + FCJ * (AG - AGC) + (FS - FCI) * AST + (FS - FCJ) * AAS
    DPC = (FCI * A4C + FCJ * (A4I - A4C) + (FS - FCI) * A4S / RECS + (FS - FCJ) * A4SS / RECS) / XXP
    goto.x30

    label.x10
    # INITIAL SECTION
    XXP = FCI * AG + (FS - FCI) * (AST + AAS)
    DPC = 0.0
    if TYPEC == 3:
        DPC = (FCI * A4I + (FS - FCI) * (A4S + A4SS) / RECS) / XXP

    label.x20
    # CONFINED SECTION
    if ITCONF == 0:
        goto.x30
    XXPC = FCI * AGC + AST * (FS - FCI)
    DPCC = 0.0
    if TYPEC == 3:
        DPCC = (FCI * A4C + (FS - FCI) * A4S / RECS) / XXPC

    # if IND != 1:
    #     if ISPALL != 0:
    #         # SECTION WITH DIFFERENT CONCRETE IN COVER AND CORE
    #         FCJ = - STRESS(ECI, ECS, FCS)
    #         XXP = FCI * AGC + FCJ * (AG - AGC) + (FS - FCI) * AST + (FS - FCJ) * AAS
    #         DPC = (FCI * A4C + FCJ * (A4I - A4C) + (FS - FCI) * A4S / RECS + (FS - FCJ) * A4SS / RECS) / XXP
    #     else:
    #         # INITIAL SECTION
    #         XXP = FCI * AG + (FS - FCI) * (AST + AAS)
    #         DPC = 0.0
    #         if TYPEC == 3:
    #             DPC = (FCI * A4I + (FS - FCI) * (A4S + A4SS) / RECS) / XXP
    #
    #         # print(f"XXP={XXP:10.3f}   DPC={DPC}")
    # # else:
    # if ISPALL == 0:
    #     # CONFINED SECTION
    #     if ITCONF != 0:
    #         XXPC = FCI * AGC + AST * (FS - FCI)
    #         DPCC = 0.0
    #         if TYPEC == 3:
    #             DPCC = (FCI * A4C + (FS - FCI) * A4S / RECS) / XXPC
    #         # print(f"XXPC={XXPC:10.3f}   DPCC={DPCC}")

    label.x30

    return XXP, DPC, XXPC, DPCC


def STRESS(ECI, EC, FC):
    _FCI = 0.
    if ECI <= 0.:
        _FCI = 0.
    else:
        _FCI = - np.interp(ECI, EC, FC)
    return _FCI


def STRHRD(EX, FYS, YMS, ESH, YSH, FSU, ESU, C1, C2):
    _FS = -YMS * EX
    FS1 = _FS
    if FS1 == 0.:
        FS1 = 0.000001
    SIGNS = _FS / abs(FS1)

    if abs(_FS) > FYS:
        _FS = FYS * SIGNS

    EXX = abs(EX)
    if EXX < ESU:
        if EXX > ESH:
            DS = EXX - ESH
            _FS = (FYS + (((C1 * DS) + C2) * DS + YSH) * DS) * SIGNS
    else:
        _FS = FSU * SIGNS

    return _FS


def CONC(ECU, B, EC, FC):
    COMP = 0.0
    ZZC = 0.0

    for i in range(0, len(EC)):
        if ECU >= EC[i + 1]:
            A1 = (FC[i] + FC[i + 1]) * (EC[i + 1] - EC[i])
            COMP = COMP + A1
            Z1 = ((EC[i + 1] - EC[i]) * (2.0 * FC[i] + FC[i + 1]) / (3.0 * (FC[i] + FC[i + 1]))+(ECU - EC[i + 1])) * A1
            ZZC = ZZC + Z1
        else:
            FCECU = FC[i] + (FC[i + 1] - FC[i]) * (ECU - EC[i]) / (EC[i + 1] - EC[i])
            A1 = (FC[i] + FCECU) * (ECU - EC[i])
            COMP = COMP + A1
            Z1 = ((ECU - EC[i]) * (2.0 * FC[i] + FCECU) / (3.0 * (FC[i] + FCECU))) * A1
            ZZC = ZZC + Z1
            break

    ZC = ZZC / (ECU * COMP)
    COMP = -0.5 * B * COMP / ECU

    return COMP, ZC


def CFRCS(C, T, ECU, A, Y, EC, FC):
    NLAYER = len(A) - 1
    XI = float(NLAYER) * C / T + 0.5
    I = int(XI)
    if I > NLAYER:
        I = NLAYER
    T1 = ECU / C
    T2 = (C - 0.5 * T) * T1

    XP = 0.0
    XM = 0.0
    for n in range(1, I+1):
        ECI = T2 + Y[n] * T1
        FCI = STRESS(ECI, EC, FC)
        T3 = A[n] * FCI
        XP = XP + T3
        XM = XM + T3 * Y[n]

    return XP, XM


def CFRCR(C, ECU, T, B, EC, FC):
    if C <= T:
        COMP, ZC = CONC(ECU, B, EC, FC)
    else:
        COMP, ZC = FLANGE(ECU, C, T, B, EC, FC)
    XP = C * COMP
    XM = XP * (T / 2. - ZC * C)

    # print(f'XP = {XP}     XM = {XM}')
    return XP, XM


@with_goto
def FLANGE(ECU, C, TT, BB, EC, FC):

    # print('FLANGE')
    COMPF = 0.0
    ZCF = 0.0
    COMP = 0.0
    ZZC = 0.0

    FCECU = - STRESS(ECU, EC, FC)

    for i in range(0, len(EC)):
        if COMP != 0.0:
            goto.x20
        ZZ = (ECU - EC[i]) * C / ECU
        if ZZ > TT:
            continue
        ECBOT = ECU * (1. - TT / C)
        FCBOT = np.interp(ECBOT, EC, FC)

        if EC[i] < ECU:
            goto.x10
        COMP = (FCBOT + FCECU) * (ECU - ECBOT)
        ZZC = ((ECU - ECBOT) * (2. * FCBOT + FCECU) / (3. * (FCBOT + FCECU))) * COMP
        break

        label.x10
        COMP = (FCBOT + FC[i]) * (EC[i] - ECBOT)
        ZZC = ((EC[i] - ECBOT) * (2. * FCBOT + FC[i]) / (3. * (FCBOT + FC[i])) + (ECU - EC[i])) * COMP

        label.x20
        if ECU < EC[i+1]:
            goto.x30
        A1 = (FC[i] + FC[i + 1]) * (EC[i + 1] - EC[i])
        COMP = COMP + A1
        Z1 = ((EC[i + 1] - EC[i]) * (2. * FC[i] + FC[i + 1]) / (3. * (FC[i] + FC[i + 1])) + (ECU - EC[i + 1])) * A1
        ZZC = ZZC + Z1
        goto.x40

        label.x30
        A1 = (FC[i] + FCECU) * (ECU - EC[i])
        COMP = COMP + A1
        Z1 = ((ECU - EC[i]) * (2. * FC[i] + FCECU) / (3. * (FC[i] + FCECU))) * A1
        ZZC = ZZC + Z1
        break

        label.x40
        continue

    label.x50
    ZCF = ZZC / (ECU * COMP)
    COMPF = -BB * COMP / (2.0 * ECU)

    return COMPF, ZCF

@with_goto
def YLDDUC(PHII, PHI2, XXMI, XXM2, ICOUNT, K, DUCT1, O):
    DUC = 0.0
    OO = 0.0
    if ICOUNT == 1:
        goto.x60
    if PHI2[K, 1] == 0.:
        goto.x10
    DUCT1[ICOUNT] = PHII[K, ICOUNT]/PHI2[K, 1]
    O[ICOUNT] = XXMI[K, ICOUNT] / XXM2[K, 1]

    label.x10
    if PHI2[K, 2] == 0.0:
        goto.x20
    DUC = PHII[K, ICOUNT] / PHI2[K, 2]
    OO = XXMI[K, ICOUNT] / XXM2[K, 2]

    label.x20
    if DUCT1[ICOUNT] > DUC:
        goto.x30
    DUCT1[ICOUNT] = DUC
    O[ICOUNT] = OO

    label.x30
    if ICOUNT != 2:
        goto.x60
    DUC = 0.0
    OO = 0.0
    if PHI2[K, 1] == 0.0:
        goto.x40
    DUCT1[1] = PHII[K, 1] / PHI2[K, 1]
    O[1] = XXMI[K, 1] / XXM2[K, 1]

    label.x40
    if PHI2[K, 2] == 0.0:
        goto.x50
    DUC = PHII[K, 1] / PHI2[K, 2]
    OO = XXMI[K, 1] / XXM2[K, 2]

    label.x50
    if DUCT1[1] > DUC:
        goto.x60
    DUCT1[1] = DUC
    O[1] = OO

    label.x60

    return DUCT1, O


@with_goto
def DUCTLE(PHI, XXM, AREA, DUCT, DUCMAX, K, N, NSTRNI, NSTRNC):
    PHIYMY_K = 0.0
    L = 4
    if N == 2:
        L = 4 + NSTRNI
    J = NSTRNI + 3
    if N == 2:
        J = NSTRNC + 3
        goto.x10

    A = PHI[K, 1]
    B = PHI[K, 2]

    if A == 0.:
        A = 100000.
    if B == 0.:
        B = 100000.

    PHIYMY_K = XXM[K, 1] * PHI[K, 1]
    PM = XXM[K, 2] * PHI[K, 2]

    if A > B:
        PHIYMY_K = PM

    label.x10
    PHIMAX = 0.0
    XMMAX = 0.0
    A = 0.0
    B = 0.0
    C = 0.0

    label.x20
    for I in range(1, 4):
        AREA[I] = 0.0
        DUCT[K, I] = 0.0

    for I in range(L, J):
        if XXM[K, I] == 0.0:
            goto.x40
        if PHI[K, I] > PHIMAX:
            PHIMAX = PHI[K, I]
        if abs(XXM[K, I]) > abs(XMMAX):
            XMMAX = XXM[K, I]
        D = PHI[K, I] - B
        A = A + 0.5 * D * (XXM[K, I] + C)
        B = PHI[K, I]
        C = XXM[K, I]
        AREA[I] = A
        if PHIYMY_K == 0.0:
            goto.x50
        DUCT[K, I] = A / PHIYMY_K + 0.5
        if DUCT[K, I] > DUCMAX:
            DUCMAX = DUCT[K, I]
        continue

        label.x40
        AREA[I] = 0.0

        label.x50
        DUCT[K, I] = 0.0

    AA = 0.0
    BB = 0.0
    CC = 0.0

    if N != 2:
        AA = A
        BB = B
        CC = C

    return DUCT, AREA, DUCMAX


@with_goto
def SHEAR(V, VM, I, K, AG, G, TL, AST, XL, XP, F, ASTL, D, BW, TYPEC, TYPES, ICONF):
    if I == 1 or V[I-1] == 0.0:
        goto.x10
    V[I] = V[I-1]
    VM[K, I] = VM[K, I-1]
    goto.x90

    label.x10
    if XP < 0.0:
        goto.x50

    if ICONF == 1:
        goto.x20
    if TYPEC == 2:
        goto.x30

    goto.x40

    label.x20
    if TYPES==2:
        goto.x30
    goto.x40

    label.x30
    A = AG * (G + 5000. * TL * AST / (XL * AG))
    C = 40.0 * XP
    E = TL * F
    AE = A + E
    V[I] = (AE + math.sqrt(AE * AE + 4.0 * A * C)) / 2.0
    goto.x80

    label.x40
    A = 2500. * ASTL * D
    B = XL / 2.0
    C = XP * (4.0 * TL - D) * 125.0
    E = BW * D * (G + F / BW)
    CABE = C + A + B * E
    V[I] = (CABE + math.sqrt(CABE * CABE - 4.0 * B * C * E)) / (2.0 * B)
    VMAX = BW * D * (F / BW + 1.842 * G * math.sqrt(1. + 0.29 * XP / AG))
    if V[I] > VMAX:
        V[I] = VMAX
    goto.x80

    label.x50
    A = 0.29 * XP / AG
    A = 1.0 + A
    if A < 0.0:
        A = 0.0
    A = 2.0 * A * G / 1.9
    AV = BW * D
    if TYPEC == 2:
        goto.x60
    if TYPES == 2 and ICONF == 1:
        goto.x60
    goto.x70

    label.x60
    R = TL / 2.0
    X = 2.0 * math.sqrt(2. * R * D - D * D)
    B = 2.0 * math.sqrt(X / (2. * R))
    if D > R:
        B = 2 * math.pi - B
    B = B * R
    AV = (B * R - X * (R - D)) / 2.0

    label.x70
    V[I] = A * AV + F * D

    label.x80
    VM[K, I] = V[I] * XL / 2000.
    V[I] = V[I] * 0.001

    label.x90
    return V, VM





from dataclasses import dataclass
import numpy as np
from sympy.solvers import solve
from sympy import Symbol
import math
from streng.common.io import results_display_older


@dataclass
class InfillPanelGK:
    h: float
    l: float
    fwc: float
    t: float
    ρ: float
    N: float
    Ac: float

    kw = None
    γcr = None
    τcr = None
    τu = None
    γu = None
    area = None
    γD = None
    τ002 = None

    kw_BC = None
    kw_CD = None
    Aw = None
    α = None
    EsAs_AB = None
    EsAs_BC = None
    EsAs_CD = None
    d = None
    dA = None
    dB = None
    dC = None
    dD = None
    dE = None
    δA = None
    δB = None
    δC = None
    δD = None
    δE = None
    FA = None
    FB = None
    FC = None
    FD = None
    FE = None

    XFδ = None
    YFδ = None

    log_list = None

    def __post_init__(self):
        self.calc()

    @staticmethod
    def τ3(γ: float, τu: float, γu: float, ρ: float, σ: float):
        x1 = (γ / γu) - 1
        x2 = ρ ** (3.82 * (1 - 0.22 * σ))
        x3 = 1.0 + 0.166 * σ
        return τu * (1 - 0.24 * np.sqrt(x1 / (x2 * x3)))


    def calc(self):

        self.log_list = list()

        self.σ = self.N * 0.001 / self.Ac

        self.kw = 1.71 * self.fwc * (80 + self.h / self.t)
        self.γcr = 0.09 / (self.fwc ** 0.5 * (80 + self.h / self.t))
        self.τcr = self.kw * self.γcr

        self.τu = 0.22 * self.fwc ** 0.5
        self.γu = 4.545 * self.γcr



        self.τ002 = self.τ3(0.02, self.τu, self.γu, self.ρ, self.σ)

        # τ-γ για τους 2 πρώτους κλάδους
        x1 = [0, self.γcr, self.γu]
        y1 = [0, self.τcr, self.τu]

        # τ-γ για τον 3ο κλάδο
        x2 = np.linspace(self.γu, 0.02, 50)
        vτ3 = np.vectorize(self.τ3)
        y2 = vτ3(x2, self.τu, self.γu, self.ρ, self.σ)

        # Εμβαδό κάτω από το διάγραμμα του 3ου κλάδου
        self.area = np.trapz(y2, x2)

        γtmp = Symbol('γtmp')
        self.γD = float(solve((self.τ002 * (0.02 - γtmp) + 0.5 * (γtmp - self.γu) * (self.τ002 + self.τu) - self.area), γtmp)[0])

        self.kw_BC = (self.τu - self.τcr) / (self.γu - self.γcr)
        self.kw_CD = (self.τ002 - self.τu) / (self.γD - self.γu)

        self.Aw = self.l * self.t
        self.α = np.arctan(self.h / self.l)  # Η γωνία με την οριζόντιο

        self.EsAs_AB = 1000 * self.Aw * self.kw / (math.sin(self.α) * math.cos(self.α) * math.cos(self.α))
        self.EsAs_BC = 1000 * self.Aw * self.kw_BC / (math.sin(self.α) * math.cos(self.α) * math.cos(self.α))
        self.EsAs_CD = 1000 * self.Aw * self.kw_CD / (math.sin(self.α) * math.cos(self.α) * math.cos(self.α))

        self.d = (self.h ** 2 + self.l ** 2) ** 0.5
        self.dA = self.d
        self.dB = math.sqrt(math.pow(self.h, 2) + math.pow(self.l - self.h * self.γcr, 2))
        self.dC = math.sqrt(math.pow(self.h, 2) + math.pow(self.l - self.h * self.γu, 2))
        self.dD = math.sqrt(math.pow(self.h, 2) + math.pow(self.l - self.h * self.γD, 2))
        self.dE = math.sqrt(math.pow(self.h, 2) + math.pow(self.l - self.h * 0.02, 2))

        self.δA = 0.0
        self.δB = - (self.dB - self.dA)
        self.δC = - (self.dC - self.dA)
        self.δD = - (self.dD - self.dA)
        self.δE = - (self.dE - self.dA)

        self.FA = 0
        self.FB = self.EsAs_AB * self.δB / self.d
        self.FC = self.EsAs_BC * (self.δC - self.δB) / self.d + self.FB
        self.FD = self.EsAs_CD * (self.δD - self.δC) / self.d + self.FC
        self.FE = self.FD

        self.log_list.append({'quantity': 'σ', 'value': self.σ, 'units': ''})
        self.log_list.append({'quantity': '1oς κλάδος', 'value':None, 'units': ''})
        self.log_list.append({'quantity': 'kw', 'value': self.kw, 'units': 'MPa'})
        self.log_list.append({'quantity': 'γcr', 'value': self.γcr, 'units': ''})
        self.log_list.append({'quantity': 'τcr', 'value': self.τcr, 'units': 'MPa'})
        self.log_list.append({'quantity': '2oς κλάδος', 'value':None, 'units': ''})
        self.log_list.append({'quantity': 'γu', 'value': self.γu, 'units': ''})
        self.log_list.append({'quantity': 'τu', 'value': self.τu, 'units': 'MPa'})

        # self.log_list.append({'quantity': 'x1', 'value': x1, 'units': ''})
        # self.log_list.append({'quantity': 'y1', 'value': y1, 'units': ''})
        # self.log_list.append({'quantity': 'x2', 'value': x2, 'units': ''})
        # self.log_list.append({'quantity': 'y2', 'value': y2, 'units': ''})

        self.log_list.append({'quantity': '3oς κλάδος', 'value':None, 'units': ''})
        self.log_list.append({'quantity': 'Εμβαδό 3ου κλάδου', 'value': self.area, 'units': ''})

        self.log_list.append({'quantity': 'γD', 'value': self.γD, 'units': ''})
        self.log_list.append({'quantity': 'τD=τ002', 'value': self.τ002, 'units': 'MPa'})

        self.log_list.append({'quantity': 'kw_BC', 'value': self.kw_BC, 'units': ''})
        self.log_list.append({'quantity': 'kw_CD', 'value': self.kw_CD, 'units': ''})

        self.log_list.append({'quantity': 'EsAs_AB', 'value': self.EsAs_AB, 'units': ''})
        self.log_list.append({'quantity': 'EsAs_BC', 'value': self.EsAs_BC, 'units': ''})
        self.log_list.append({'quantity': 'EsAs_CD', 'value': self.EsAs_CD, 'units': ''})

        self.log_list.append({'quantity': 'dA', 'value': self.dA, 'units': ''})
        self.log_list.append({'quantity': 'dB', 'value': self.dB, 'units': ''})
        self.log_list.append({'quantity': 'dC', 'value': self.dC, 'units': ''})
        self.log_list.append({'quantity': 'dD', 'value': self.dD, 'units': ''})
        self.log_list.append({'quantity': 'dE', 'value': self.dE, 'units': ''})

        self.log_list.append({'quantity': 'δA', 'value': self.δA, 'units': ''})
        self.log_list.append({'quantity': 'δB', 'value': self.δB, 'units': ''})
        self.log_list.append({'quantity': 'δC', 'value': self.δC, 'units': ''})
        self.log_list.append({'quantity': 'δD', 'value': self.δD, 'units': ''})
        self.log_list.append({'quantity': 'δE', 'value': self.δE, 'units': ''})



        self.log_list.append({'quantity': 'FA', 'value': self.FA, 'units': ''})
        self.log_list.append({'quantity': 'FB', 'value': self.FB, 'units': ''})
        self.log_list.append({'quantity': 'FC', 'value': self.FC, 'units': ''})
        self.log_list.append({'quantity': 'FD', 'value': self.FD, 'units': ''})
        self.log_list.append({'quantity': 'FE', 'value': self.FE, 'units': ''})

        self.XFδ = [self.δA, self.δB, self.δC, self.δD, self.δE]
        self.YFδ = [self.FA, self.FB, self.FC, self.FD, self.FE]


    def log_panda(self):
        return results_display_older.log_dataframe(self.log_list)

    def log_markdown_table(self):
        return results_display_older.log_markdown_table(self.log_list)

    def __str__(self):
        return self.log_markdown_table()



import pandas as pd
from tabulate import tabulate


# def log_dataframe(log_table, columns=('quantity', 'value', 'units')):
#     res = pd.DataFrame(columns=columns)
#     for item in log_table:
#         dicts = dict(zip(columns, item))
#         res = res.append([dicts], ignore_index=True)
# #         res = res.append([{'quantity': item[0], 'value': item[1], 'units': item[2]}], ignore_index=True)
#     return res

# def log_formatted_table(log_table, headers=None, table_format="pipe", float_fmt=".3E"):
#     if headers is None:
#         headers = ['quantity', 'value', 'units']
#     ftm_tbl = tabulate(log_table, headers, tablefmt=table_format, floatfmt=float_fmt)
#     return ftm_tbl

def log_dataframe(list_of_dicts):
    res = pd.DataFrame(list_of_dicts,
                       columns=list(list_of_dicts.keys()))
                       # columns=list(list_of_dicts[0].keys()))
    return res

def log_markdown_table(list_of_dicts, table_format="pipe", float_fmt=".3E"):
    ftm_tbl = tabulate(list_of_dicts,
                       headers='keys',
                       tablefmt=table_format,
                       floatfmt=float_fmt)
    return ftm_tbl

def log_retrieve(list_of_dicts, quantity):
    res = [d['value'] for d in list_of_dicts if d['quantity']==quantity]
    return res[0]


import pandas as pd
from . import s2k_from_pandas as sap_from_pd
from . import tcl_from_pandas as tcl_from_pd


class Input:

    def __init__(self, input_filetype, filename):
        self.__s2k_file = ''
        self.__tcl_file = ''

        if input_filetype == 'excel':
            self.df_nodes = pd.read_excel(filename, sheet_name='nodes')
            self.df_node_restraints = pd.read_excel(filename, sheet_name='node_restraints')
            self.df_node_constraints = pd.read_excel(filename, sheet_name='node_constraints')
            self.df_materials_general = pd.read_excel(filename, sheet_name='materials_general')
            self.df_frame_connectivity = pd.read_excel(filename, sheet_name='frame_connectivity')
            self.df_section_rect_elastic = pd.read_excel(filename, sheet_name='section_rect_elastic')
            self.df_load_patterns = pd.read_excel(filename, sheet_name='load_patterns')
            self.df_masses_nodal = pd.read_excel(filename, sheet_name='masses_nodal')
            self.df_loads_nodal = pd.read_excel(filename, sheet_name='loads_nodal')
            self.df_loads_dist_gravity = pd.read_excel(filename, sheet_name='loads_dist_gravity')

    @staticmethod
    def __save_file(filename, text):
        file = open(filename, 'w')
        file.write(text)
        file.close()

    def make_sap2d_s2k_file(self):
        _str = 'TABLE:  "PROGRAM CONTROL"\n'
        _str += '   ProgramName=SAP2000   Version=14.2.4   CurrUnits="KN, m, C"   RegenHinge=Yes"\n'
        _str += '\n'
        _str += sap_from_pd.sap2d_nodes(self.df_nodes) 
        _str += sap_from_pd.sap2d_gridlines(self.df_nodes) 
        _str += sap_from_pd.sap2d_diaphragms(self.df_node_constraints) 
        _str += sap_from_pd.sap2d_diaphragms_assignments(self.df_node_constraints) 
        _str += sap_from_pd.sap2d_node_restraints(self.df_node_restraints) 
        _str += sap_from_pd.sap2d_frame_connectivity(self.df_frame_connectivity) 
        _str += sap_from_pd.sap2d_materials_general(self.df_materials_general) 
        _str += sap_from_pd.sap2d_materials_basic_mechanical(self.df_materials_general) 
        _str += sap_from_pd.sap2d_frame_section_assignments(self.df_frame_connectivity, self.df_section_rect_elastic)
        _str += sap_from_pd.sap2d_frame_section_properties_general(self.df_section_rect_elastic) 
        _str += sap_from_pd.sap2d_load_patterns(self.df_load_patterns) 
        _str += sap_from_pd.sap2d_load_cases(self.df_load_patterns) 
        _str += sap_from_pd.sap2d_load_cases_static_assignments(self.df_load_patterns) 
        _str += sap_from_pd.sap2d_masses() 
        _str += sap_from_pd.sap2d_masses_nodal(self.df_masses_nodal) 
        _str += sap_from_pd.sap2d_loads_nodal(self.df_loads_nodal) 
        _str += sap_from_pd.sap2d_loads_dist(self.df_loads_dist_gravity)
        self.__s2k_file = _str

    @property
    def s2k_file(self):
        if self.__s2k_file == '':
            return 'You should call "make_sap2d_s2k_file" first'
        else:
            return self.__s2k_file

    def save_s2k_file(self, filename):
        self.__save_file(filename, self.__s2k_file)

    def make_os2d_tcl_file(self):
        _str = ''
        _str += tcl_from_pd.os2d_general()
        _str += tcl_from_pd.os2d_nodes(self.df_nodes)
        _str += tcl_from_pd.os2d_node_restraints(self.df_node_restraints)
        _str += tcl_from_pd.os2d_diaphragms(self.df_node_constraints)
        _str += tcl_from_pd.os2d_masses_nodal(self.df_masses_nodal)
        _str += tcl_from_pd.os2d_frame_connectivity(self.df_frame_connectivity,
                                                    self.df_section_rect_elastic,
                                                    self.df_materials_general)
        self.__tcl_file = _str

    @property
    def tcl_file(self):
        if self.__tcl_file == '':
            return 'You should call "make_os2d_tcl_file" first'
        else:
            return self.__tcl_file

    def save_tcl_file(self, filename):
        self.__save_file(filename, self.__tcl_file)


import math
from tabulate import tabulate
from ...raw.ch7a import yield_point as yp

from dataclasses import dataclass

# @dataclass
# class YieldPoint:



# def yield_props(ρ1, ρ2, ρv, N, b, d, δtonos, fc, Ec, fy, Es, makelog=False, makelatex=False):
#     logtext = ''
#     latext = ''
#
#     α = Es/Ec
#     Asteel = ρ1 + ρ2 + ρv + N / (b * d * fy)
#     Bsteel = ρ1 + ρ2 * δtonos + 0.5 * ρv * (1 + δtonos) + N / (b * d * fy)
#     ξysteel = ξycalc(α, Asteel, Bsteel, makelatex)
#     φysteel = fy / (Es * (1.0 - ξysteel[0]) * d)
#     Aconc = ρ1 + ρ2 + ρv - N / (1.8 * α * b * d * fc)
#     Bconc = ρ1 + ρ2 * δtonos + 0.5 * ρv * (1 + δtonos)
#     ξyconc = ξycalc(α, Aconc, Bconc, makelatex)
#     φyconc = 1.8 * fc / (Ec * ξyconc[0] * d)
#
#     if φysteel<φyconc:
#         φy = φysteel
#         ξy = ξysteel[0]
#     else:
#         φy = φyconc
#         ξy = ξyconc[0]
#
#     if makelog:
#         headers = ['', 'value', 'units']
#         table = [['α', α],
#                  ['Asteel', Asteel],
#                  ['Bsteel', Bsteel],
#                  ['ξysteel', ξysteel[0], 'm'],
#                  ['φysteel', φysteel, 'm-1'],
#                  ['Aconc', Aconc],
#                  ['Bconc', Bconc],
#                  ['ξyconc', ξyconc[0]],
#                  ['φyconc', φyconc, 'm-1'],
#                  ['φy', φy, 'm-1'],
#                  ['ξy', ξy, 'm']]
#         logtext = tabulate(table, headers, tablefmt="pipe", floatfmt=".3E")
#
#
#     return α, Asteel, Bsteel, ξysteel[0], φysteel, Aconc, Bconc, ξyconc[0], φyconc, φy, ξy, logtext, latext


import numpy as np


def T1(ground_type):
    """

    Args:
        ground_type (str): Ground type (Α, Β, Γ, or Δ)

    Returns:
        float: The lower limit of the period of the constant spectral acceleration branch

    """
    data = {'Α': 0.10, 'Β': 0.15, 'Γ': 0.20, 'Δ': 0.20}
    return data[ground_type]


def T2(ground_type):
    """

        ground_type (str): Ground type (Α, Β, Γ, or Δ)

    Returns:
        float: The upper limit of the period of the constant spectral acceleration branch

    """
    data = {'Α': 0.40, 'Β': 0.60, 'Γ': 0.80, 'Δ': 1.20}
    return data[ground_type]


def Φd(T, α, γI,  T1, T2, q, η=1.0, θ=1.0, β0=2.5):
    """

    Args:
        T (float): The vibration period of a linear single-degree-of-freedom system
        α (float): The design ground acceleration (A=α*g)
        γI (float): Importance factor (0.85, 1.0, 1.15, 1.30 for Σ1, Σ2, Σ3 and Σ4)
        T1 (float): The lower limit of the period of the constant spectral acceleration branch
        T2 (float): The upper limit of the period of the constant spectral acceleration branch
        q (float): The behaviour factor
        η (float): Correction factor when damping is not 5%
        θ (float): Foundation effect factor
        β0 (float): Spectral amplification factor. Recommended value for β0 is 2.5

    Returns:
        float: Design spectrum. Given using the expressions:

        .. math::
            :nowrap:

            \\begin{eqnarray}
                coming \\\\
                soon \\\\
            \\end{eqnarray}

    """
    condlist = [T <= T1,
                T <= T2,
                T <= 4]
    choicelist = [γI * α * (1+(T/T1)*(η * θ * β0 / q - 1)),
                  γI * α * (η * θ * β0 / q),
                  γI * α * (η * θ * β0 / q) * (T2/T)**(2/3)]
    return np.select(condlist, choicelist)


def η(ζ):
    """

    Args:
        ζ (float): the viscous damping ratio of the structure[%]

    Returns:
        float: The value of the damping correction factor. Given using the expressions:

        .. math::
            η = \sqrt{\dfrac{7}{2+ζ}} \ge 0.55
    """

    return (7. / (2. + ζ)) ** 0.5

